import time, socket

ZEROCONF_TIMEOUT_DEFAULT = 4

class ZeroconfDiscoveryListener:
    def __init__(self):
        self.found_pdus = {}
        self.pdu_prefixes = ["PX2", "PX3", "SRC", "BCM", "BCM2", "PX3TS", "PMC", "PMMC", "PXE", "SCH", "SRC", "LP", "PXO", "PXC", "EMX"]

    def remove_service(self, zeroconf, service_type, name):
        pass

    def add_service(self, zeroconf, service_type, name):
        try:
            entry = zeroconf.get_service_info(service_type, name)
            if not any([entry.name.startswith(pfx) for pfx in self.pdu_prefixes]):
                return
            self.found_pdus[entry.addresses[0]] = { "fw": entry.properties[b'rr_fw'], "port": entry.port }
        except (KeyboardInterrupt, ImportError, NameError):
            pass

def discover(timeout = ZEROCONF_TIMEOUT_DEFAULT):
    try:
        from zeroconf import ServiceBrowser, Zeroconf
        zeroconf = Zeroconf()
        listener = ZeroconfDiscoveryListener()
        browser_old_name = ServiceBrowser(zeroconf, "_raritan_rpcs._tcp.local.", listener)
        browser_new_name = ServiceBrowser(zeroconf, "_raritan-rpcs._tcp.local.", listener)
        print("Zeroconf discovery started")
        time.sleep(timeout)
        browser_old_name.cancel()
        browser_new_name.cancel()
        zeroconf.close()
        pdus = []
        print("Found %i PDUs in the local subnet:" % len(listener.found_pdus))
        for k, v in listener.found_pdus.items():
            ip = str(socket.inet_ntoa(k))
            port = v.get("port")
            fw_version = str(v.get("fw"), "utf-8")
            print("%s -- %s" % (ip, fw_version))
            pdus.append({ "ip": ip, "port": port, "fw": fw_version })
        return pdus
    except ImportError:
        print("Please install the zeroconf python module (pip3 install zeroconf)")
        raise



# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright 2020 Raritan Inc. All rights reserved.
#
# This is an auto-generated file.

#
# Section generated by IdlC from "SessionManager.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.session


# structure
class Session(Structure):
    idlType = "session.Session:2.0.0"
    elements = ["sessionId", "username", "remoteIp", "clientType", "creationTime", "timeout", "idle", "userIdle"]

    def __init__(self, sessionId, username, remoteIp, clientType, creationTime, timeout, idle, userIdle):
        typecheck.is_int(sessionId, AssertionError)
        typecheck.is_string(username, AssertionError)
        typecheck.is_string(remoteIp, AssertionError)
        typecheck.is_string(clientType, AssertionError)
        typecheck.is_time(creationTime, AssertionError)
        typecheck.is_int(timeout, AssertionError)
        typecheck.is_int(idle, AssertionError)
        typecheck.is_int(userIdle, AssertionError)

        self.sessionId = sessionId
        self.username = username
        self.remoteIp = remoteIp
        self.clientType = clientType
        self.creationTime = creationTime
        self.timeout = timeout
        self.idle = idle
        self.userIdle = userIdle

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            sessionId = json['sessionId'],
            username = json['username'],
            remoteIp = json['remoteIp'],
            clientType = json['clientType'],
            creationTime = raritan.rpc.Time.decode(json['creationTime']),
            timeout = json['timeout'],
            idle = json['idle'],
            userIdle = json['userIdle'],
        )
        return obj

    def encode(self):
        json = {}
        json['sessionId'] = self.sessionId
        json['username'] = self.username
        json['remoteIp'] = self.remoteIp
        json['clientType'] = self.clientType
        json['creationTime'] = raritan.rpc.Time.encode(self.creationTime)
        json['timeout'] = self.timeout
        json['idle'] = self.idle
        json['userIdle'] = self.userIdle
        return json

# structure
class HistoryEntry(Structure):
    idlType = "session.HistoryEntry:1.0.0"
    elements = ["creationTime", "remoteIp", "clientType"]

    def __init__(self, creationTime, remoteIp, clientType):
        typecheck.is_time(creationTime, AssertionError)
        typecheck.is_string(remoteIp, AssertionError)
        typecheck.is_string(clientType, AssertionError)

        self.creationTime = creationTime
        self.remoteIp = remoteIp
        self.clientType = clientType

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            creationTime = raritan.rpc.Time.decode(json['creationTime']),
            remoteIp = json['remoteIp'],
            clientType = json['clientType'],
        )
        return obj

    def encode(self):
        json = {}
        json['creationTime'] = raritan.rpc.Time.encode(self.creationTime)
        json['remoteIp'] = self.remoteIp
        json['clientType'] = self.clientType
        return json

# interface
class SessionManager(Interface):
    idlType = "session.SessionManager:2.0.0"

    ERR_ACTIVE_SESSION_EXCLUSIVE_FOR_USER = 1

    # enumeration
    class CloseReason(Enumeration):
        idlType = "session.SessionManager_2_0_0.CloseReason:1.0.0"
        values = ["CLOSE_REASON_LOGOUT", "CLOSE_REASON_TIMEOUT", "CLOSE_REASON_BROWSER_CLOSED", "CLOSE_REASON_FORCED_DISCONNECT"]

    CloseReason.CLOSE_REASON_LOGOUT = CloseReason(0)
    CloseReason.CLOSE_REASON_TIMEOUT = CloseReason(1)
    CloseReason.CLOSE_REASON_BROWSER_CLOSED = CloseReason(2)
    CloseReason.CLOSE_REASON_FORCED_DISCONNECT = CloseReason(3)

    class _newSession(Interface.Method):
        name = 'newSession'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            session = raritan.rpc.session.Session.decode(rsp['session'], agent)
            token = rsp['token']
            typecheck.is_int(_ret_, DecodeException)
            typecheck.is_struct(session, raritan.rpc.session.Session, DecodeException)
            typecheck.is_string(token, DecodeException)
            return (_ret_, session, token)

    class _getCurrentSession(Interface.Method):
        name = 'getCurrentSession'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.session.Session.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.session.Session, DecodeException)
            return _ret_

    class _getSessions(Interface.Method):
        name = 'getSessions'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = [raritan.rpc.session.Session.decode(x0, agent) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_struct(x0, raritan.rpc.session.Session, DecodeException)
            return _ret_

    class _closeSession(Interface.Method):
        name = 'closeSession'

        @staticmethod
        def encode(sessionId, reason):
            typecheck.is_int(sessionId, AssertionError)
            typecheck.is_enum(reason, raritan.rpc.session.SessionManager.CloseReason, AssertionError)
            args = {}
            args['sessionId'] = sessionId
            args['reason'] = raritan.rpc.session.SessionManager.CloseReason.encode(reason)
            return args

        @staticmethod
        def decode(rsp, agent):
            return None

    class _closeCurrentSession(Interface.Method):
        name = 'closeCurrentSession'

        @staticmethod
        def encode(reason):
            typecheck.is_enum(reason, raritan.rpc.session.SessionManager.CloseReason, AssertionError)
            args = {}
            args['reason'] = raritan.rpc.session.SessionManager.CloseReason.encode(reason)
            return args

        @staticmethod
        def decode(rsp, agent):
            return None

    class _touchCurrentSession(Interface.Method):
        name = 'touchCurrentSession'

        @staticmethod
        def encode(userActivity):
            typecheck.is_bool(userActivity, AssertionError)
            args = {}
            args['userActivity'] = userActivity
            return args

        @staticmethod
        def decode(rsp, agent):
            return None

    class _getSessionHistory(Interface.Method):
        name = 'getSessionHistory'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = [raritan.rpc.session.HistoryEntry.decode(x0, agent) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_struct(x0, raritan.rpc.session.HistoryEntry, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(SessionManager, self).__init__(target, agent)
        self.newSession = SessionManager._newSession(self)
        self.getCurrentSession = SessionManager._getCurrentSession(self)
        self.getSessions = SessionManager._getSessions(self)
        self.closeSession = SessionManager._closeSession(self)
        self.closeCurrentSession = SessionManager._closeCurrentSession(self)
        self.touchCurrentSession = SessionManager._touchCurrentSession(self)
        self.getSessionHistory = SessionManager._getSessionHistory(self)



# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright 2010 Raritan Inc. All rights reserved.

# Avoid name clash with raritan.rpc.sys
from __future__ import absolute_import

import base64, json, ssl, sys, uuid
import raritan.rpc

try:
    # Python 3
    import urllib.request as urllib_request
except ImportError:
    # Python 2
    import urllib2 as urllib_request

class Agent(object):
    """Provides transport to one RPC service, e.g. one PX2 device - holds host,
       user name, and password."""
    id = 1

    def __init__(self, proto, host, user = None, passwd = None, token = None,
                 debug = False, disable_certificate_verification = False, timeout = None):
        self.url = "%s://%s" % (proto, host)
        self.user = user
        self.passwd = passwd
        self.token = token # authentication token
        self.debug = debug
        self.timeout = timeout

        context = None
        if disable_certificate_verification:
            import ssl
            if "_create_unverified_context" in ssl.__dict__.keys():
                context = ssl._create_unverified_context()

        self.opener = urllib_request.OpenerDirector()
        self.opener.add_handler(urllib_request.HTTPHandler())
        try:
            self.opener.add_handler(urllib_request.HTTPSHandler(context = context))
        except TypeError:
            # Python < 2.7.9
            self.opener.add_handler(urllib_request.HTTPSHandler())

        Agent.defaultInst = self

    def __create_request(self, target_url, data = None):
        Agent.id += 1

        if data != None:
            request = urllib_request.Request(target_url, data)
        else:
            request = urllib_request.Request(target_url)

        if self.token != None:
            request.add_header("X-SessionToken", self.token)
        elif self.user != None and self.passwd != None:
            basic = base64.b64encode(str.encode('%s:%s' % (self.user, self.passwd)))
            request.add_header('Authorization', 'Basic ' + bytes.decode(basic))
        return request

    def __open_request(self, request):
        try:
            if (self.timeout):
                response = self.opener.open(request, timeout = self.timeout)
            else:
                response = self.opener.open(request)

        except IOError as e:
            if str(e).find("CERTIFICATE_VERIFY_FAILED") >= 0:
                sys.stderr.write("==================================================================\n")
                sys.stderr.write(" SSL certificate verification failed!\n")
                sys.stderr.write("\n")
                sys.stderr.write(" When connecting to a device without valid SSL certificate, try\n")
                sys.stderr.write(" adding 'disable_certificate_verification=True' when creating the\n")
                sys.stderr.write(" raritan.rpc.Agent instance.\n")
                sys.stderr.write("==================================================================\n")
            raise raritan.rpc.HttpException("Opening URL %s failed: %s" % (request.get_full_url(), e))
        return response

    def set_auth_basic(self, user, passwd):
        self.user = user
        self.passwd = passwd
        self.token = None

    def set_auth_token(self, token):
        self.user = None
        self.passwd = None
        self.token = token

    def handle_http_redirect(self, rid, response):
        location = response.headers["Location"]
        baselen = len(location) - len(rid)
        if baselen <= 0:
            return False
        elif location[baselen:] != rid:
            return False
        else:
            self.url = location[:baselen]
            if self.debug:
                print("Redirected to: " + self.url)
            return True

    def get(self, target, redirected = False):
        target_url = "%s/%s" % (self.url, target)
        request = self.__create_request(target_url)
        response = self.__open_request(request)

        if response.code == 302 and not redirected:
            # handle HTTP-to-HTTPS redirect and try again
            if self.handle_http_redirect(target, response):
                return self.get(target, True)

        # get and process response
        try:
            resp = response.read()
        except:
            raise raritan.rpc.HttpException("Reading response failed.")

        if response.code != 200:
            raise raritan.rpc.HttpException("HTTP Error %d\nResponse:\n%s" % (response.code, str(resp)))

        if (self.debug):
            print("download: Response:\n%s" % str(resp))

        return resp


    def form_data_file(self, target, datas, filenames, formnames, mimetypes, redirected = False):
        target_url = "%s/%s" % (self.url, target)
        request = self.__create_request(target_url)

        boundary = uuid.uuid4().hex
        # for certificate use key_file and cert_file
        bodyArr = []
        for i in range(len(filenames)):
            data = datas[i]
            filename = filenames[i]
            formname = formnames[i]
            mimetype = mimetypes[i]
            bodyArr.append('--%s' % boundary)
            bodyArr.append('Content-Disposition: form-data; name="%s"; filename="%s"' % (formname, filename))
            bodyArr.append('Content-Type: %s' % mimetype)
            bodyArr.append('')
            bodyArr.append(data)
        bodyArr.append('--%s--' % boundary)
        body = bytes()
        for l in bodyArr:
            if isinstance(l, bytes): body += l + b'\r\n'
            else: body += bytes(l, encoding='utf8') + b'\r\n'

        request.add_header('Content-Type', 'multipart/form-data; boundary=%s' % boundary)
        try:
            request.data = body
        except AttributeError:
            request.add_data(body)

        response = self.__open_request(request)

        if response.code == 302 and not redirected:
            # handle HTTP-to-HTTPS redirect and try again
            if self.handle_http_redirect(target, response):
                return self.form_data_file(target, datas, filenames, mimetypes, True)

        # get and process response
        try:
            resp = bytes.decode(response.read())
        except:
            raise raritan.rpc.HttpException("Reading response failed.")

        if response.code != 200:
            raise raritan.rpc.HttpException("HTTP Error %d\nResponse:\n%s" % (response.code, resp))

        if (self.debug):
            print("form_data: Response:\n%s" % resp)

        return response


    def json_rpc(self, target, method, params = [], redirected = False):
        request_json = json.dumps({"method": method, "params": params, "id": Agent.id})
        if (self.debug):
            print("json_rpc: %s() - %s: , request = %s" % (method, target, request_json))

        target_url = "%s/%s" % (self.url, target)
        request = self.__create_request(target_url, str.encode(request_json))

        if self.token != None:
            request.add_header("X-SessionToken", self.token)
        elif self.user != None and self.passwd != None:
            basic = base64.b64encode(str.encode('%s:%s' % (self.user, self.passwd)))
            request.add_header('Authorization', 'Basic ' + bytes.decode(basic))

        response = self.__open_request(request)

        if response.code == 302 and not redirected:
            # handle HTTP-to-HTTPS redirect and try again
            if self.handle_http_redirect(target, response):
                return self.json_rpc(target, method, params, True)

        # get and process response
        try:
            resp = bytes.decode(response.read())
        except:
            raise raritan.rpc.HttpException("Reading response failed.")

        if response.code != 200:
            raise raritan.rpc.HttpException("HTTP Error %d\nResponse:\n%s" % (response.code, resp))

        if (self.debug):
            print("json_rpc: Response:\n%s" % resp)

        try:
            resp_json = json.loads(resp)
        except ValueError as e:
            raise raritan.rpc.JsonRpcSyntaxException(
                    "Decoding response to JSON failed: %s" % e)

        if "error" in resp_json:
            try:
                code = resp_json["error"]["code"]
                msg = resp_json["error"]["message"]
            except KeyError:
                raise raritan.rpc.JsonRpcSyntaxException(
                        "JSON RPC returned malformed error: %s" % resp_json)
            raise raritan.rpc.JsonRpcErrorException(
                    "JSON RPC returned error: code = %d, msg = %s" % (code, msg))

        try:
            res = resp_json["result"]
        except KeyError:
            raise raritan.rpc.JsonRpcSyntaxException(
                    "Result is missing in JSON RPC response: %s" % resp_json)

        return res



# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright 2020 Raritan Inc. All rights reserved.
#
# This is an auto-generated file.

#
# Section generated by IdlC from "Firmware.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.event

import raritan.rpc.firmware

import raritan.rpc.idl


# enumeration
class UpdateHistoryStatus(Enumeration):
    idlType = "firmware.UpdateHistoryStatus:1.0.0"
    values = ["SUCCESSFUL", "FAILED", "INCOMPLETE"]

UpdateHistoryStatus.SUCCESSFUL = UpdateHistoryStatus(0)
UpdateHistoryStatus.FAILED = UpdateHistoryStatus(1)
UpdateHistoryStatus.INCOMPLETE = UpdateHistoryStatus(2)

# structure
class UpdateHistoryEntry(Structure):
    idlType = "firmware.UpdateHistoryEntry:1.0.0"
    elements = ["timestamp", "oldVersion", "imageVersion", "imageMD5", "status"]

    def __init__(self, timestamp, oldVersion, imageVersion, imageMD5, status):
        typecheck.is_time(timestamp, AssertionError)
        typecheck.is_string(oldVersion, AssertionError)
        typecheck.is_string(imageVersion, AssertionError)
        typecheck.is_string(imageMD5, AssertionError)
        typecheck.is_enum(status, raritan.rpc.firmware.UpdateHistoryStatus, AssertionError)

        self.timestamp = timestamp
        self.oldVersion = oldVersion
        self.imageVersion = imageVersion
        self.imageMD5 = imageMD5
        self.status = status

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            timestamp = raritan.rpc.Time.decode(json['timestamp']),
            oldVersion = json['oldVersion'],
            imageVersion = json['imageVersion'],
            imageMD5 = json['imageMD5'],
            status = raritan.rpc.firmware.UpdateHistoryStatus.decode(json['status']),
        )
        return obj

    def encode(self):
        json = {}
        json['timestamp'] = raritan.rpc.Time.encode(self.timestamp)
        json['oldVersion'] = self.oldVersion
        json['imageVersion'] = self.imageVersion
        json['imageMD5'] = self.imageMD5
        json['status'] = raritan.rpc.firmware.UpdateHistoryStatus.encode(self.status)
        return json

# enumeration
class ImageState(Enumeration):
    idlType = "firmware.ImageState:1.0.0"
    values = ["NONE", "UPLOADING", "UPLOAD_FAILED", "DOWNLOADING", "DOWNLOAD_FAILED", "COMPLETE"]

ImageState.NONE = ImageState(0)
ImageState.UPLOADING = ImageState(1)
ImageState.UPLOAD_FAILED = ImageState(2)
ImageState.DOWNLOADING = ImageState(3)
ImageState.DOWNLOAD_FAILED = ImageState(4)
ImageState.COMPLETE = ImageState(5)

# structure
class ImageStatus(Structure):
    idlType = "firmware.ImageStatus:1.0.0"
    elements = ["state", "error_message", "time_started", "size_total", "size_done"]

    def __init__(self, state, error_message, time_started, size_total, size_done):
        typecheck.is_enum(state, raritan.rpc.firmware.ImageState, AssertionError)
        typecheck.is_string(error_message, AssertionError)
        typecheck.is_time(time_started, AssertionError)
        typecheck.is_int(size_total, AssertionError)
        typecheck.is_int(size_done, AssertionError)

        self.state = state
        self.error_message = error_message
        self.time_started = time_started
        self.size_total = size_total
        self.size_done = size_done

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            state = raritan.rpc.firmware.ImageState.decode(json['state']),
            error_message = json['error_message'],
            time_started = raritan.rpc.Time.decode(json['time_started']),
            size_total = json['size_total'],
            size_done = json['size_done'],
        )
        return obj

    def encode(self):
        json = {}
        json['state'] = raritan.rpc.firmware.ImageState.encode(self.state)
        json['error_message'] = self.error_message
        json['time_started'] = raritan.rpc.Time.encode(self.time_started)
        json['size_total'] = self.size_total
        json['size_done'] = self.size_done
        return json

# structure
class ImageInfo(Structure):
    idlType = "firmware.ImageInfo:1.0.1"
    elements = ["valid", "version", "min_required_version", "min_downgrade_version", "product", "platform", "oem", "hwid_whitelist", "hwid_blacklist", "compatible", "signature_present", "signed_by", "signature_good", "certified_by", "certificate_good", "model_list_present", "model_supported"]

    def __init__(self, valid, version, min_required_version, min_downgrade_version, product, platform, oem, hwid_whitelist, hwid_blacklist, compatible, signature_present, signed_by, signature_good, certified_by, certificate_good, model_list_present, model_supported):
        typecheck.is_bool(valid, AssertionError)
        typecheck.is_string(version, AssertionError)
        typecheck.is_string(min_required_version, AssertionError)
        typecheck.is_string(min_downgrade_version, AssertionError)
        typecheck.is_string(product, AssertionError)
        typecheck.is_string(platform, AssertionError)
        typecheck.is_string(oem, AssertionError)
        typecheck.is_string(hwid_whitelist, AssertionError)
        typecheck.is_string(hwid_blacklist, AssertionError)
        typecheck.is_bool(compatible, AssertionError)
        typecheck.is_bool(signature_present, AssertionError)
        typecheck.is_string(signed_by, AssertionError)
        typecheck.is_bool(signature_good, AssertionError)
        typecheck.is_string(certified_by, AssertionError)
        typecheck.is_bool(certificate_good, AssertionError)
        typecheck.is_bool(model_list_present, AssertionError)
        typecheck.is_bool(model_supported, AssertionError)

        self.valid = valid
        self.version = version
        self.min_required_version = min_required_version
        self.min_downgrade_version = min_downgrade_version
        self.product = product
        self.platform = platform
        self.oem = oem
        self.hwid_whitelist = hwid_whitelist
        self.hwid_blacklist = hwid_blacklist
        self.compatible = compatible
        self.signature_present = signature_present
        self.signed_by = signed_by
        self.signature_good = signature_good
        self.certified_by = certified_by
        self.certificate_good = certificate_good
        self.model_list_present = model_list_present
        self.model_supported = model_supported

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            valid = json['valid'],
            version = json['version'],
            min_required_version = json['min_required_version'],
            min_downgrade_version = json['min_downgrade_version'],
            product = json['product'],
            platform = json['platform'],
            oem = json['oem'],
            hwid_whitelist = json['hwid_whitelist'],
            hwid_blacklist = json['hwid_blacklist'],
            compatible = json['compatible'],
            signature_present = json['signature_present'],
            signed_by = json['signed_by'],
            signature_good = json['signature_good'],
            certified_by = json['certified_by'],
            certificate_good = json['certificate_good'],
            model_list_present = json['model_list_present'],
            model_supported = json['model_supported'],
        )
        return obj

    def encode(self):
        json = {}
        json['valid'] = self.valid
        json['version'] = self.version
        json['min_required_version'] = self.min_required_version
        json['min_downgrade_version'] = self.min_downgrade_version
        json['product'] = self.product
        json['platform'] = self.platform
        json['oem'] = self.oem
        json['hwid_whitelist'] = self.hwid_whitelist
        json['hwid_blacklist'] = self.hwid_blacklist
        json['compatible'] = self.compatible
        json['signature_present'] = self.signature_present
        json['signed_by'] = self.signed_by
        json['signature_good'] = self.signature_good
        json['certified_by'] = self.certified_by
        json['certificate_good'] = self.certificate_good
        json['model_list_present'] = self.model_list_present
        json['model_supported'] = self.model_supported
        return json

# enumeration
class UpdateFlags(Enumeration):
    idlType = "firmware.UpdateFlags:1.0.0"
    values = ["CROSS_OEM", "CROSS_HW", "ALLOW_UNTRUSTED"]

UpdateFlags.CROSS_OEM = UpdateFlags(0)
UpdateFlags.CROSS_HW = UpdateFlags(1)
UpdateFlags.ALLOW_UNTRUSTED = UpdateFlags(2)

# value object
class SystemStartupEvent(raritan.rpc.idl.Event):
    idlType = "firmware.SystemStartupEvent:1.0.0"

    def __init__(self, source):
        super(raritan.rpc.firmware.SystemStartupEvent, self).__init__(source)

    def encode(self):
        json = super(raritan.rpc.firmware.SystemStartupEvent, self).encode()
        return json

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            # for idl.Event
            source = Interface.decode(json['source'], agent),
        )
        return obj

    def listElements(self):
        elements = []
        elements = elements + super(raritan.rpc.firmware.SystemStartupEvent, self).listElements()
        return elements

# value object
class SystemShutdownEvent(raritan.rpc.event.UserEvent):
    idlType = "firmware.SystemShutdownEvent:1.0.0"

    def __init__(self, actUserName, actIpAddr, source):
        super(raritan.rpc.firmware.SystemShutdownEvent, self).__init__(actUserName, actIpAddr, source)

    def encode(self):
        json = super(raritan.rpc.firmware.SystemShutdownEvent, self).encode()
        return json

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            # for event.UserEvent
            actUserName = json['actUserName'],
            actIpAddr = json['actIpAddr'],
            # for idl.Event
            source = Interface.decode(json['source'], agent),
        )
        return obj

    def listElements(self):
        elements = []
        elements = elements + super(raritan.rpc.firmware.SystemShutdownEvent, self).listElements()
        return elements

# value object
class FirmwareValidationFailedEvent(raritan.rpc.event.UserEvent):
    idlType = "firmware.FirmwareValidationFailedEvent:1.0.0"

    def __init__(self, actUserName, actIpAddr, source):
        super(raritan.rpc.firmware.FirmwareValidationFailedEvent, self).__init__(actUserName, actIpAddr, source)

    def encode(self):
        json = super(raritan.rpc.firmware.FirmwareValidationFailedEvent, self).encode()
        return json

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            # for event.UserEvent
            actUserName = json['actUserName'],
            actIpAddr = json['actIpAddr'],
            # for idl.Event
            source = Interface.decode(json['source'], agent),
        )
        return obj

    def listElements(self):
        elements = []
        elements = elements + super(raritan.rpc.firmware.FirmwareValidationFailedEvent, self).listElements()
        return elements

# value object
class FirmwareUpdateEvent(raritan.rpc.event.UserEvent):
    idlType = "firmware.FirmwareUpdateEvent:1.0.0"

    def __init__(self, oldVersion, newVersion, actUserName, actIpAddr, source):
        super(raritan.rpc.firmware.FirmwareUpdateEvent, self).__init__(actUserName, actIpAddr, source)
        typecheck.is_string(oldVersion, AssertionError)
        typecheck.is_string(newVersion, AssertionError)

        self.oldVersion = oldVersion
        self.newVersion = newVersion

    def encode(self):
        json = super(raritan.rpc.firmware.FirmwareUpdateEvent, self).encode()
        json['oldVersion'] = self.oldVersion
        json['newVersion'] = self.newVersion
        return json

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            oldVersion = json['oldVersion'],
            newVersion = json['newVersion'],
            # for event.UserEvent
            actUserName = json['actUserName'],
            actIpAddr = json['actIpAddr'],
            # for idl.Event
            source = Interface.decode(json['source'], agent),
        )
        return obj

    def listElements(self):
        elements = ["oldVersion", "newVersion"]
        elements = elements + super(raritan.rpc.firmware.FirmwareUpdateEvent, self).listElements()
        return elements

# value object
class FirmwareUpdateStartedEvent(FirmwareUpdateEvent):
    idlType = "firmware.FirmwareUpdateStartedEvent:1.0.0"

    def __init__(self, oldVersion, newVersion, actUserName, actIpAddr, source):
        super(raritan.rpc.firmware.FirmwareUpdateStartedEvent, self).__init__(oldVersion, newVersion, actUserName, actIpAddr, source)

    def encode(self):
        json = super(raritan.rpc.firmware.FirmwareUpdateStartedEvent, self).encode()
        return json

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            # for firmware.FirmwareUpdateEvent
            oldVersion = json['oldVersion'],
            newVersion = json['newVersion'],
            # for event.UserEvent
            actUserName = json['actUserName'],
            actIpAddr = json['actIpAddr'],
            # for idl.Event
            source = Interface.decode(json['source'], agent),
        )
        return obj

    def listElements(self):
        elements = []
        elements = elements + super(raritan.rpc.firmware.FirmwareUpdateStartedEvent, self).listElements()
        return elements

# value object
class FirmwareUpdateCompletedEvent(FirmwareUpdateEvent):
    idlType = "firmware.FirmwareUpdateCompletedEvent:1.0.0"

    def __init__(self, oldVersion, newVersion, actUserName, actIpAddr, source):
        super(raritan.rpc.firmware.FirmwareUpdateCompletedEvent, self).__init__(oldVersion, newVersion, actUserName, actIpAddr, source)

    def encode(self):
        json = super(raritan.rpc.firmware.FirmwareUpdateCompletedEvent, self).encode()
        return json

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            # for firmware.FirmwareUpdateEvent
            oldVersion = json['oldVersion'],
            newVersion = json['newVersion'],
            # for event.UserEvent
            actUserName = json['actUserName'],
            actIpAddr = json['actIpAddr'],
            # for idl.Event
            source = Interface.decode(json['source'], agent),
        )
        return obj

    def listElements(self):
        elements = []
        elements = elements + super(raritan.rpc.firmware.FirmwareUpdateCompletedEvent, self).listElements()
        return elements

# value object
class FirmwareUpdateFailedEvent(FirmwareUpdateEvent):
    idlType = "firmware.FirmwareUpdateFailedEvent:1.0.0"

    def __init__(self, oldVersion, newVersion, actUserName, actIpAddr, source):
        super(raritan.rpc.firmware.FirmwareUpdateFailedEvent, self).__init__(oldVersion, newVersion, actUserName, actIpAddr, source)

    def encode(self):
        json = super(raritan.rpc.firmware.FirmwareUpdateFailedEvent, self).encode()
        return json

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            # for firmware.FirmwareUpdateEvent
            oldVersion = json['oldVersion'],
            newVersion = json['newVersion'],
            # for event.UserEvent
            actUserName = json['actUserName'],
            actIpAddr = json['actIpAddr'],
            # for idl.Event
            source = Interface.decode(json['source'], agent),
        )
        return obj

    def listElements(self):
        elements = []
        elements = elements + super(raritan.rpc.firmware.FirmwareUpdateFailedEvent, self).listElements()
        return elements

# interface
class Firmware(Interface):
    idlType = "firmware.Firmware:2.0.1"

    class _reboot(Interface.Method):
        name = 'reboot'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            return None

    class _factoryReset(Interface.Method):
        name = 'factoryReset'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            return None

    class _hardFactoryReset(Interface.Method):
        name = 'hardFactoryReset'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _getVersion(Interface.Method):
        name = 'getVersion'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_string(_ret_, DecodeException)
            return _ret_

    class _getUpdateHistory(Interface.Method):
        name = 'getUpdateHistory'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = [raritan.rpc.firmware.UpdateHistoryEntry.decode(x0, agent) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_struct(x0, raritan.rpc.firmware.UpdateHistoryEntry, DecodeException)
            return _ret_

    class _getImageStatus(Interface.Method):
        name = 'getImageStatus'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.firmware.ImageStatus.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.firmware.ImageStatus, DecodeException)
            return _ret_

    class _discardImage(Interface.Method):
        name = 'discardImage'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            return None

    class _getImageInfo(Interface.Method):
        name = 'getImageInfo'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            info = raritan.rpc.firmware.ImageInfo.decode(rsp['info'], agent)
            typecheck.is_bool(_ret_, DecodeException)
            typecheck.is_struct(info, raritan.rpc.firmware.ImageInfo, DecodeException)
            return (_ret_, info)

    class _startUpdate(Interface.Method):
        name = 'startUpdate'

        @staticmethod
        def encode(flags):
            for x0 in flags:
                typecheck.is_enum(x0, raritan.rpc.firmware.UpdateFlags, AssertionError)
            args = {}
            args['flags'] = [raritan.rpc.firmware.UpdateFlags.encode(x0) for x0 in flags]
            return args

        @staticmethod
        def decode(rsp, agent):
            return None
    def __init__(self, target, agent):
        super(Firmware, self).__init__(target, agent)
        self.reboot = Firmware._reboot(self)
        self.factoryReset = Firmware._factoryReset(self)
        self.hardFactoryReset = Firmware._hardFactoryReset(self)
        self.getVersion = Firmware._getVersion(self)
        self.getUpdateHistory = Firmware._getUpdateHistory(self)
        self.getImageStatus = Firmware._getImageStatus(self)
        self.discardImage = Firmware._discardImage(self)
        self.getImageInfo = Firmware._getImageInfo(self)
        self.startUpdate = Firmware._startUpdate(self)

#
# Section generated by IdlC from "FirmwareUpdateStatus.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.firmware


# structure
class UpdateStatus(Structure):
    idlType = "firmware.UpdateStatus:1.0.0"
    elements = ["state", "elapsed", "estimated", "error_message"]

    def __init__(self, state, elapsed, estimated, error_message):
        typecheck.is_string(state, AssertionError)
        typecheck.is_int(elapsed, AssertionError)
        typecheck.is_int(estimated, AssertionError)
        typecheck.is_string(error_message, AssertionError)

        self.state = state
        self.elapsed = elapsed
        self.estimated = estimated
        self.error_message = error_message

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            state = json['state'],
            elapsed = json['elapsed'],
            estimated = json['estimated'],
            error_message = json['error_message'],
        )
        return obj

    def encode(self):
        json = {}
        json['state'] = self.state
        json['elapsed'] = self.elapsed
        json['estimated'] = self.estimated
        json['error_message'] = self.error_message
        return json

# interface
class FirmwareUpdateStatus(Interface):
    idlType = "firmware.FirmwareUpdateStatus:1.0.0"

    class _getStatus(Interface.Method):
        name = 'getStatus'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.firmware.UpdateStatus.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.firmware.UpdateStatus, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(FirmwareUpdateStatus, self).__init__(target, agent)
        self.getStatus = FirmwareUpdateStatus._getStatus(self)

# from raritan/rpc/firmware/__extend__.py
def upload(agent, data):
    """
    Method to upload firmware images

    **parameters**, **return**

    :param agent: An agent instance for the device where the image should be uploaded
    :param data: The binary data of the firmware image

    **Example**
        :Example:

        from raritan import rpc
        from raritan.rpc import firmware

        agent = rpc.Agent("https", "my-pdu.example.com", "admin", "raritan")
        firmware_proxy = firmware.Firmware("/firmware", agent)

        # read file in binary mode
        fwFile = open("pdu-030600-45660.bin", "rb")
        # upload
        firmware.upload(agent, fwFile.read())

    """
    target = "cgi-bin/fwupload.cgi"
    agent.form_data_file(target, [data], ["upfile.bin"], ["upfile"], ["application/octet-stream"])



# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright 2020 Raritan Inc. All rights reserved.
#
# This is an auto-generated file.

#
# Section generated by IdlC from "RawConfiguration.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.rawcfg


# interface
class RawConfiguration(Interface):
    idlType = "rawcfg.RawConfiguration:1.0.0"

    # enumeration
    class Status(Enumeration):
        idlType = "rawcfg.RawConfiguration.Status:1.0.0"
        values = ["UNKNOWN", "UPLOAD_FAILED", "UPDATE_PENDING", "UPDATE_OK", "UPDATE_FAILED"]

    Status.UNKNOWN = Status(0)
    Status.UPLOAD_FAILED = Status(1)
    Status.UPDATE_PENDING = Status(2)
    Status.UPDATE_OK = Status(3)
    Status.UPDATE_FAILED = Status(4)

    class _getStatus(Interface.Method):
        name = 'getStatus'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            status = raritan.rpc.rawcfg.RawConfiguration.Status.decode(rsp['status'])
            timeStamp = raritan.rpc.Time.decode(rsp['timeStamp'])
            typecheck.is_enum(status, raritan.rpc.rawcfg.RawConfiguration.Status, DecodeException)
            typecheck.is_time(timeStamp, DecodeException)
            return (status, timeStamp)
    def __init__(self, target, agent):
        super(RawConfiguration, self).__init__(target, agent)
        self.getStatus = RawConfiguration._getStatus(self)

# from raritan/rpc/rawcfg/__extend__.py
def upload(agent, data):
    """
    Method to upload raw config files

    **parameters**, **return**

    :param agent: An agent instance for the device where the config should be uploaded
    :param data: The binary data of the raw config file
    :return: return upload response

    **Example**
        :Example:

        from raritan import rpc
        from raritan.rpc import rawcfg

        agent = rpc.Agent("https", "my-pdu.example.com", "admin", "raritan")

        # read file in binary mode
        cfgFile = open("config.txt", "rb")
        # upload
        code = rawcfg.upload(agent, cfgFile.read())
        # view code
        print(code)

    """
    target = "cgi-bin/raw_config_update.cgi"
    response = agent.form_data_file(target, [data], ["config.txt"], ["config_file"], ["application/octet-stream"])
    return response.headers.get("X-Response-Code")

def download_rawcfg(agent):
    """
    Method to download the configuration data

    **parameters**

    :param agent: An agent instance from the device where the raw configuration data should be downloaded
    :return: returns the raw configuration data

    **Example**
        :Example:

        from raritan import rpc
        from raritan.rpc import rawcfg

        agent = rpc.Agent("https", "my-pdu.example.com", "admin", "raritan")
        # download
        raw_cfg = rawcfg.download_rawcfg(agent)
        print(raw_cfg)
    """
    target = "cgi-bin/raw_config_download.cgi"
    return agent.get(target)



# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright 2020 Raritan Inc. All rights reserved.
#
# This is an auto-generated file.

#
# Section generated by IdlC from "Webcam.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.event

import raritan.rpc.webcam


# enumeration
class PixelFormat(Enumeration):
    idlType = "webcam.PixelFormat:1.0.0"
    values = ["MJPEG", "JPEG", "RGB", "YUV"]

PixelFormat.MJPEG = PixelFormat(0)
PixelFormat.JPEG = PixelFormat(1)
PixelFormat.RGB = PixelFormat(2)
PixelFormat.YUV = PixelFormat(3)

# enumeration
class PowerLineFrequency(Enumeration):
    idlType = "webcam.PowerLineFrequency:1.0.0"
    values = ["NOT_SUPPORTED", "HZ50", "HZ60", "DISABLED"]

PowerLineFrequency.NOT_SUPPORTED = PowerLineFrequency(0)
PowerLineFrequency.HZ50 = PowerLineFrequency(1)
PowerLineFrequency.HZ60 = PowerLineFrequency(2)
PowerLineFrequency.DISABLED = PowerLineFrequency(3)

# structure
class Format(Structure):
    idlType = "webcam.Format:2.0.0"
    elements = ["width", "height", "pixelFormat"]

    def __init__(self, width, height, pixelFormat):
        typecheck.is_int(width, AssertionError)
        typecheck.is_int(height, AssertionError)
        typecheck.is_enum(pixelFormat, raritan.rpc.webcam.PixelFormat, AssertionError)

        self.width = width
        self.height = height
        self.pixelFormat = pixelFormat

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            width = json['width'],
            height = json['height'],
            pixelFormat = raritan.rpc.webcam.PixelFormat.decode(json['pixelFormat']),
        )
        return obj

    def encode(self):
        json = {}
        json['width'] = self.width
        json['height'] = self.height
        json['pixelFormat'] = raritan.rpc.webcam.PixelFormat.encode(self.pixelFormat)
        return json

# structure
class Controls(Structure):
    idlType = "webcam.Controls:1.0.0"
    elements = ["brightness", "contrast", "saturation", "gain", "gamma", "powerLineFrequency"]

    def __init__(self, brightness, contrast, saturation, gain, gamma, powerLineFrequency):
        typecheck.is_int(brightness, AssertionError)
        typecheck.is_int(contrast, AssertionError)
        typecheck.is_int(saturation, AssertionError)
        typecheck.is_int(gain, AssertionError)
        typecheck.is_int(gamma, AssertionError)
        typecheck.is_enum(powerLineFrequency, raritan.rpc.webcam.PowerLineFrequency, AssertionError)

        self.brightness = brightness
        self.contrast = contrast
        self.saturation = saturation
        self.gain = gain
        self.gamma = gamma
        self.powerLineFrequency = powerLineFrequency

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            brightness = json['brightness'],
            contrast = json['contrast'],
            saturation = json['saturation'],
            gain = json['gain'],
            gamma = json['gamma'],
            powerLineFrequency = raritan.rpc.webcam.PowerLineFrequency.decode(json['powerLineFrequency']),
        )
        return obj

    def encode(self):
        json = {}
        json['brightness'] = self.brightness
        json['contrast'] = self.contrast
        json['saturation'] = self.saturation
        json['gain'] = self.gain
        json['gamma'] = self.gamma
        json['powerLineFrequency'] = raritan.rpc.webcam.PowerLineFrequency.encode(self.powerLineFrequency)
        return json

# structure
class Location(Structure):
    idlType = "webcam.Location:1.0.0"
    elements = ["name", "x", "y", "z"]

    def __init__(self, name, x, y, z):
        typecheck.is_string(name, AssertionError)
        typecheck.is_string(x, AssertionError)
        typecheck.is_string(y, AssertionError)
        typecheck.is_string(z, AssertionError)

        self.name = name
        self.x = x
        self.y = y
        self.z = z

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            name = json['name'],
            x = json['x'],
            y = json['y'],
            z = json['z'],
        )
        return obj

    def encode(self):
        json = {}
        json['name'] = self.name
        json['x'] = self.x
        json['y'] = self.y
        json['z'] = self.z
        return json

# structure
class ImageMetaData(Structure):
    idlType = "webcam.ImageMetaData:1.0.0"
    elements = ["format", "timestamp", "location"]

    def __init__(self, format, timestamp, location):
        typecheck.is_struct(format, raritan.rpc.webcam.Format, AssertionError)
        typecheck.is_long(timestamp, AssertionError)
        typecheck.is_struct(location, raritan.rpc.webcam.Location, AssertionError)

        self.format = format
        self.timestamp = timestamp
        self.location = location

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            format = raritan.rpc.webcam.Format.decode(json['format'], agent),
            timestamp = int(json['timestamp']),
            location = raritan.rpc.webcam.Location.decode(json['location'], agent),
        )
        return obj

    def encode(self):
        json = {}
        json['format'] = raritan.rpc.webcam.Format.encode(self.format)
        json['timestamp'] = self.timestamp
        json['location'] = raritan.rpc.webcam.Location.encode(self.location)
        return json

# structure
class Image(Structure):
    idlType = "webcam.Image:2.0.0"
    elements = ["meta", "data"]

    def __init__(self, meta, data):
        typecheck.is_struct(meta, raritan.rpc.webcam.ImageMetaData, AssertionError)
        typecheck.is_string(data, AssertionError)

        self.meta = meta
        self.data = data

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            meta = raritan.rpc.webcam.ImageMetaData.decode(json['meta'], agent),
            data = json['data'],
        )
        return obj

    def encode(self):
        json = {}
        json['meta'] = raritan.rpc.webcam.ImageMetaData.encode(self.meta)
        json['data'] = self.data
        return json

# structure
class Settings(Structure):
    idlType = "webcam.Settings:2.0.0"
    elements = ["format", "controls", "name", "location", "refreshInterval"]

    def __init__(self, format, controls, name, location, refreshInterval):
        typecheck.is_struct(format, raritan.rpc.webcam.Format, AssertionError)
        typecheck.is_struct(controls, raritan.rpc.webcam.Controls, AssertionError)
        typecheck.is_string(name, AssertionError)
        typecheck.is_struct(location, raritan.rpc.webcam.Location, AssertionError)
        typecheck.is_int(refreshInterval, AssertionError)

        self.format = format
        self.controls = controls
        self.name = name
        self.location = location
        self.refreshInterval = refreshInterval

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            format = raritan.rpc.webcam.Format.decode(json['format'], agent),
            controls = raritan.rpc.webcam.Controls.decode(json['controls'], agent),
            name = json['name'],
            location = raritan.rpc.webcam.Location.decode(json['location'], agent),
            refreshInterval = json['refreshInterval'],
        )
        return obj

    def encode(self):
        json = {}
        json['format'] = raritan.rpc.webcam.Format.encode(self.format)
        json['controls'] = raritan.rpc.webcam.Controls.encode(self.controls)
        json['name'] = self.name
        json['location'] = raritan.rpc.webcam.Location.encode(self.location)
        json['refreshInterval'] = self.refreshInterval
        return json

# structure
class Information(Structure):
    idlType = "webcam.Information:3.0.0"
    elements = ["id", "model", "supportedFormats"]

    def __init__(self, id, model, supportedFormats):
        typecheck.is_string(id, AssertionError)
        typecheck.is_string(model, AssertionError)
        for x0 in supportedFormats:
            typecheck.is_struct(x0, raritan.rpc.webcam.Format, AssertionError)

        self.id = id
        self.model = model
        self.supportedFormats = supportedFormats

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            id = json['id'],
            model = json['model'],
            supportedFormats = [raritan.rpc.webcam.Format.decode(x0, agent) for x0 in json['supportedFormats']],
        )
        return obj

    def encode(self):
        json = {}
        json['id'] = self.id
        json['model'] = self.model
        json['supportedFormats'] = [raritan.rpc.webcam.Format.encode(x0) for x0 in self.supportedFormats]
        return json

# interface
class Webcam(Interface):
    idlType = "webcam.Webcam:2.0.1"

    # value object
    class SettingsChangedEvent(raritan.rpc.event.UserEvent):
        idlType = "webcam.Webcam_2_0_1.SettingsChangedEvent:1.0.0"

        def __init__(self, oldSettings, newSettings, actUserName, actIpAddr, source):
            super(raritan.rpc.webcam.Webcam.SettingsChangedEvent, self).__init__(actUserName, actIpAddr, source)
            typecheck.is_struct(oldSettings, raritan.rpc.webcam.Settings, AssertionError)
            typecheck.is_struct(newSettings, raritan.rpc.webcam.Settings, AssertionError)

            self.oldSettings = oldSettings
            self.newSettings = newSettings

        def encode(self):
            json = super(raritan.rpc.webcam.Webcam.SettingsChangedEvent, self).encode()
            json['oldSettings'] = raritan.rpc.webcam.Settings.encode(self.oldSettings)
            json['newSettings'] = raritan.rpc.webcam.Settings.encode(self.newSettings)
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                oldSettings = raritan.rpc.webcam.Settings.decode(json['oldSettings'], agent),
                newSettings = raritan.rpc.webcam.Settings.decode(json['newSettings'], agent),
                # for event.UserEvent
                actUserName = json['actUserName'],
                actIpAddr = json['actIpAddr'],
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["oldSettings", "newSettings"]
            elements = elements + super(raritan.rpc.webcam.Webcam.SettingsChangedEvent, self).listElements()
            return elements

    class _getInformation(Interface.Method):
        name = 'getInformation'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.webcam.Information.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.webcam.Information, DecodeException)
            return _ret_

    class _getSettings(Interface.Method):
        name = 'getSettings'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.webcam.Settings.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.webcam.Settings, DecodeException)
            return _ret_

    class _setSettings(Interface.Method):
        name = 'setSettings'

        @staticmethod
        def encode(settings):
            typecheck.is_struct(settings, raritan.rpc.webcam.Settings, AssertionError)
            args = {}
            args['settings'] = raritan.rpc.webcam.Settings.encode(settings)
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _setControls(Interface.Method):
        name = 'setControls'

        @staticmethod
        def encode(controls):
            typecheck.is_struct(controls, raritan.rpc.webcam.Controls, AssertionError)
            args = {}
            args['controls'] = raritan.rpc.webcam.Controls.encode(controls)
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _getControlDefaults(Interface.Method):
        name = 'getControlDefaults'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.webcam.Controls.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.webcam.Controls, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(Webcam, self).__init__(target, agent)
        self.getInformation = Webcam._getInformation(self)
        self.getSettings = Webcam._getSettings(self)
        self.setSettings = Webcam._setSettings(self)
        self.setControls = Webcam._setControls(self)
        self.getControlDefaults = Webcam._getControlDefaults(self)

#
# Section generated by IdlC from "StorageManager.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.event

import raritan.rpc.webcam


# interface
class StorageManager(Interface):
    idlType = "webcam.StorageManager:1.0.2"

    NO_ERROR = 0

    ERR_INVALID_PARAM = 1

    ERR_INIT_IN_PROGRESS = 2

    ERR_ALREADY_RUNNING = 3

    ERR_TOO_LARGE = 4

    ERR_OPERATION_NOT_SUPPORTED = 5

    # enumeration
    class StorageType(Enumeration):
        idlType = "webcam.StorageManager_1_0_2.StorageType:1.0.0"
        values = ["LOCAL", "FTP", "CIFS", "NFS"]

    StorageType.LOCAL = StorageType(0)
    StorageType.FTP = StorageType(1)
    StorageType.CIFS = StorageType(2)
    StorageType.NFS = StorageType(3)

    # enumeration
    class Direction(Enumeration):
        idlType = "webcam.StorageManager_1_0_2.Direction:1.0.0"
        values = ["ASCENDING", "DESCENDING"]

    Direction.ASCENDING = Direction(0)
    Direction.DESCENDING = Direction(1)

    # enumeration
    class StorageStatus(Enumeration):
        idlType = "webcam.StorageManager_1_0_2.StorageStatus:1.0.0"
        values = ["INITIALIZING", "READY"]

    StorageStatus.INITIALIZING = StorageStatus(0)
    StorageStatus.READY = StorageStatus(1)

    # structure
    class WebcamStorageInfo(Structure):
        idlType = "webcam.StorageManager_1_0_2.WebcamStorageInfo:1.0.0"
        elements = ["webcam", "newestIndex", "oldestIndex", "count"]

        def __init__(self, webcam, newestIndex, oldestIndex, count):
            typecheck.is_interface(webcam, raritan.rpc.webcam.Webcam, AssertionError)
            typecheck.is_long(newestIndex, AssertionError)
            typecheck.is_long(oldestIndex, AssertionError)
            typecheck.is_int(count, AssertionError)

            self.webcam = webcam
            self.newestIndex = newestIndex
            self.oldestIndex = oldestIndex
            self.count = count

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                webcam = Interface.decode(json['webcam'], agent),
                newestIndex = int(json['newestIndex']),
                oldestIndex = int(json['oldestIndex']),
                count = json['count'],
            )
            return obj

        def encode(self):
            json = {}
            json['webcam'] = Interface.encode(self.webcam)
            json['newestIndex'] = self.newestIndex
            json['oldestIndex'] = self.oldestIndex
            json['count'] = self.count
            return json

    # structure
    class StorageInformation(Structure):
        idlType = "webcam.StorageManager_1_0_2.StorageInformation:1.0.0"
        elements = ["status", "capacity", "used", "webcamStorageInfo"]

        def __init__(self, status, capacity, used, webcamStorageInfo):
            typecheck.is_enum(status, raritan.rpc.webcam.StorageManager.StorageStatus, AssertionError)
            typecheck.is_int(capacity, AssertionError)
            typecheck.is_int(used, AssertionError)
            for x0 in webcamStorageInfo:
                typecheck.is_struct(x0, raritan.rpc.webcam.StorageManager.WebcamStorageInfo, AssertionError)

            self.status = status
            self.capacity = capacity
            self.used = used
            self.webcamStorageInfo = webcamStorageInfo

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                status = raritan.rpc.webcam.StorageManager.StorageStatus.decode(json['status']),
                capacity = json['capacity'],
                used = json['used'],
                webcamStorageInfo = [raritan.rpc.webcam.StorageManager.WebcamStorageInfo.decode(x0, agent) for x0 in json['webcamStorageInfo']],
            )
            return obj

        def encode(self):
            json = {}
            json['status'] = raritan.rpc.webcam.StorageManager.StorageStatus.encode(self.status)
            json['capacity'] = self.capacity
            json['used'] = self.used
            json['webcamStorageInfo'] = [raritan.rpc.webcam.StorageManager.WebcamStorageInfo.encode(x0) for x0 in self.webcamStorageInfo]
            return json

    # structure
    class StorageSettings(Structure):
        idlType = "webcam.StorageManager_1_0_2.StorageSettings:1.0.0"
        elements = ["type", "capacity", "server", "username", "password"]

        def __init__(self, type, capacity, server, username, password):
            typecheck.is_enum(type, raritan.rpc.webcam.StorageManager.StorageType, AssertionError)
            typecheck.is_int(capacity, AssertionError)
            typecheck.is_string(server, AssertionError)
            typecheck.is_string(username, AssertionError)
            typecheck.is_string(password, AssertionError)

            self.type = type
            self.capacity = capacity
            self.server = server
            self.username = username
            self.password = password

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                type = raritan.rpc.webcam.StorageManager.StorageType.decode(json['type']),
                capacity = json['capacity'],
                server = json['server'],
                username = json['username'],
                password = json['password'],
            )
            return obj

        def encode(self):
            json = {}
            json['type'] = raritan.rpc.webcam.StorageManager.StorageType.encode(self.type)
            json['capacity'] = self.capacity
            json['server'] = self.server
            json['username'] = self.username
            json['password'] = self.password
            return json

    # structure
    class StorageMetaData(Structure):
        idlType = "webcam.StorageManager_1_0_2.StorageMetaData:1.0.0"
        elements = ["index", "webcam"]

        def __init__(self, index, webcam):
            typecheck.is_long(index, AssertionError)
            typecheck.is_interface(webcam, raritan.rpc.webcam.Webcam, AssertionError)

            self.index = index
            self.webcam = webcam

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                index = int(json['index']),
                webcam = Interface.decode(json['webcam'], agent),
            )
            return obj

        def encode(self):
            json = {}
            json['index'] = self.index
            json['webcam'] = Interface.encode(self.webcam)
            return json

    # structure
    class ImageStorageMetaData(Structure):
        idlType = "webcam.StorageManager_1_0_2.ImageStorageMetaData:1.0.0"
        elements = ["imageMeta", "fileSize", "storageMeta"]

        def __init__(self, imageMeta, fileSize, storageMeta):
            typecheck.is_struct(imageMeta, raritan.rpc.webcam.ImageMetaData, AssertionError)
            typecheck.is_int(fileSize, AssertionError)
            typecheck.is_struct(storageMeta, raritan.rpc.webcam.StorageManager.StorageMetaData, AssertionError)

            self.imageMeta = imageMeta
            self.fileSize = fileSize
            self.storageMeta = storageMeta

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                imageMeta = raritan.rpc.webcam.ImageMetaData.decode(json['imageMeta'], agent),
                fileSize = json['fileSize'],
                storageMeta = raritan.rpc.webcam.StorageManager.StorageMetaData.decode(json['storageMeta'], agent),
            )
            return obj

        def encode(self):
            json = {}
            json['imageMeta'] = raritan.rpc.webcam.ImageMetaData.encode(self.imageMeta)
            json['fileSize'] = self.fileSize
            json['storageMeta'] = raritan.rpc.webcam.StorageManager.StorageMetaData.encode(self.storageMeta)
            return json

    # structure
    class StorageImage(Structure):
        idlType = "webcam.StorageManager_1_0_2.StorageImage:1.0.0"
        elements = ["image", "metaData"]

        def __init__(self, image, metaData):
            typecheck.is_struct(image, raritan.rpc.webcam.Image, AssertionError)
            typecheck.is_struct(metaData, raritan.rpc.webcam.StorageManager.StorageMetaData, AssertionError)

            self.image = image
            self.metaData = metaData

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                image = raritan.rpc.webcam.Image.decode(json['image'], agent),
                metaData = raritan.rpc.webcam.StorageManager.StorageMetaData.decode(json['metaData'], agent),
            )
            return obj

        def encode(self):
            json = {}
            json['image'] = raritan.rpc.webcam.Image.encode(self.image)
            json['metaData'] = raritan.rpc.webcam.StorageManager.StorageMetaData.encode(self.metaData)
            return json

    # structure
    class Activity(Structure):
        idlType = "webcam.StorageManager_1_0_2.Activity:1.0.0"
        elements = ["webcam", "interval", "count", "done"]

        def __init__(self, webcam, interval, count, done):
            typecheck.is_interface(webcam, raritan.rpc.webcam.Webcam, AssertionError)
            typecheck.is_int(interval, AssertionError)
            typecheck.is_int(count, AssertionError)
            typecheck.is_int(done, AssertionError)

            self.webcam = webcam
            self.interval = interval
            self.count = count
            self.done = done

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                webcam = Interface.decode(json['webcam'], agent),
                interval = json['interval'],
                count = json['count'],
                done = json['done'],
            )
            return obj

        def encode(self):
            json = {}
            json['webcam'] = Interface.encode(self.webcam)
            json['interval'] = self.interval
            json['count'] = self.count
            json['done'] = self.done
            return json

    # value object
    class ImageUploadStartedEvent(raritan.rpc.event.UserEvent):
        idlType = "webcam.StorageManager_1_0_2.ImageUploadStartedEvent:1.0.0"

        def __init__(self, webcam, folderUrl, actUserName, actIpAddr, source):
            super(raritan.rpc.webcam.StorageManager.ImageUploadStartedEvent, self).__init__(actUserName, actIpAddr, source)
            typecheck.is_interface(webcam, raritan.rpc.webcam.Webcam, AssertionError)
            typecheck.is_string(folderUrl, AssertionError)

            self.webcam = webcam
            self.folderUrl = folderUrl

        def encode(self):
            json = super(raritan.rpc.webcam.StorageManager.ImageUploadStartedEvent, self).encode()
            json['webcam'] = Interface.encode(self.webcam)
            json['folderUrl'] = self.folderUrl
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                webcam = Interface.decode(json['webcam'], agent),
                folderUrl = json['folderUrl'],
                # for event.UserEvent
                actUserName = json['actUserName'],
                actIpAddr = json['actIpAddr'],
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["webcam", "folderUrl"]
            elements = elements + super(raritan.rpc.webcam.StorageManager.ImageUploadStartedEvent, self).listElements()
            return elements

    class _getSupportedStorageTypes(Interface.Method):
        name = 'getSupportedStorageTypes'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = [raritan.rpc.webcam.StorageManager.StorageType.decode(x0) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_enum(x0, raritan.rpc.webcam.StorageManager.StorageType, DecodeException)
            return _ret_

    class _getInformation(Interface.Method):
        name = 'getInformation'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.webcam.StorageManager.StorageInformation.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.webcam.StorageManager.StorageInformation, DecodeException)
            return _ret_

    class _getSettings(Interface.Method):
        name = 'getSettings'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.webcam.StorageManager.StorageSettings.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.webcam.StorageManager.StorageSettings, DecodeException)
            return _ret_

    class _setSettings(Interface.Method):
        name = 'setSettings'

        @staticmethod
        def encode(settings):
            typecheck.is_struct(settings, raritan.rpc.webcam.StorageManager.StorageSettings, AssertionError)
            args = {}
            args['settings'] = raritan.rpc.webcam.StorageManager.StorageSettings.encode(settings)
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _addImage(Interface.Method):
        name = 'addImage'

        @staticmethod
        def encode(webcam, image):
            typecheck.is_interface(webcam, raritan.rpc.webcam.Webcam, AssertionError)
            typecheck.is_struct(image, raritan.rpc.webcam.Image, AssertionError)
            args = {}
            args['webcam'] = Interface.encode(webcam)
            args['image'] = raritan.rpc.webcam.Image.encode(image)
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            index = int(rsp['index'])
            typecheck.is_int(_ret_, DecodeException)
            typecheck.is_long(index, DecodeException)
            return (_ret_, index)

    class _removeImages(Interface.Method):
        name = 'removeImages'

        @staticmethod
        def encode(webcam, start, count, direction):
            typecheck.is_interface(webcam, raritan.rpc.webcam.Webcam, AssertionError)
            typecheck.is_long(start, AssertionError)
            typecheck.is_int(count, AssertionError)
            typecheck.is_enum(direction, raritan.rpc.webcam.StorageManager.Direction, AssertionError)
            args = {}
            args['webcam'] = Interface.encode(webcam)
            args['start'] = start
            args['count'] = count
            args['direction'] = raritan.rpc.webcam.StorageManager.Direction.encode(direction)
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _getMetaData(Interface.Method):
        name = 'getMetaData'

        @staticmethod
        def encode(webcam, start, count, direction):
            typecheck.is_interface(webcam, raritan.rpc.webcam.Webcam, AssertionError)
            typecheck.is_long(start, AssertionError)
            typecheck.is_int(count, AssertionError)
            typecheck.is_enum(direction, raritan.rpc.webcam.StorageManager.Direction, AssertionError)
            args = {}
            args['webcam'] = Interface.encode(webcam)
            args['start'] = start
            args['count'] = count
            args['direction'] = raritan.rpc.webcam.StorageManager.Direction.encode(direction)
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            meta = [raritan.rpc.webcam.StorageManager.ImageStorageMetaData.decode(x0, agent) for x0 in rsp['meta']]
            typecheck.is_int(_ret_, DecodeException)
            for x0 in meta:
                typecheck.is_struct(x0, raritan.rpc.webcam.StorageManager.ImageStorageMetaData, DecodeException)
            return (_ret_, meta)

    class _getImages(Interface.Method):
        name = 'getImages'

        @staticmethod
        def encode(webcam, start, count, direction):
            typecheck.is_interface(webcam, raritan.rpc.webcam.Webcam, AssertionError)
            typecheck.is_long(start, AssertionError)
            typecheck.is_int(count, AssertionError)
            typecheck.is_enum(direction, raritan.rpc.webcam.StorageManager.Direction, AssertionError)
            args = {}
            args['webcam'] = Interface.encode(webcam)
            args['start'] = start
            args['count'] = count
            args['direction'] = raritan.rpc.webcam.StorageManager.Direction.encode(direction)
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            image = [raritan.rpc.webcam.StorageManager.StorageImage.decode(x0, agent) for x0 in rsp['image']]
            typecheck.is_int(_ret_, DecodeException)
            for x0 in image:
                typecheck.is_struct(x0, raritan.rpc.webcam.StorageManager.StorageImage, DecodeException)
            return (_ret_, image)

    class _getActivities(Interface.Method):
        name = 'getActivities'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = [raritan.rpc.webcam.StorageManager.Activity.decode(x0, agent) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_struct(x0, raritan.rpc.webcam.StorageManager.Activity, DecodeException)
            return _ret_

    class _startActivity(Interface.Method):
        name = 'startActivity'

        @staticmethod
        def encode(webcam, count, interval):
            typecheck.is_interface(webcam, raritan.rpc.webcam.Webcam, AssertionError)
            typecheck.is_int(count, AssertionError)
            typecheck.is_int(interval, AssertionError)
            args = {}
            args['webcam'] = Interface.encode(webcam)
            args['count'] = count
            args['interval'] = interval
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _startActivityWithFolder(Interface.Method):
        name = 'startActivityWithFolder'

        @staticmethod
        def encode(webcam, count, interval, folder):
            typecheck.is_interface(webcam, raritan.rpc.webcam.Webcam, AssertionError)
            typecheck.is_int(count, AssertionError)
            typecheck.is_int(interval, AssertionError)
            typecheck.is_string(folder, AssertionError)
            args = {}
            args['webcam'] = Interface.encode(webcam)
            args['count'] = count
            args['interval'] = interval
            args['folder'] = folder
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _stopActivity(Interface.Method):
        name = 'stopActivity'

        @staticmethod
        def encode(webcam):
            typecheck.is_interface(webcam, raritan.rpc.webcam.Webcam, AssertionError)
            args = {}
            args['webcam'] = Interface.encode(webcam)
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(StorageManager, self).__init__(target, agent)
        self.getSupportedStorageTypes = StorageManager._getSupportedStorageTypes(self)
        self.getInformation = StorageManager._getInformation(self)
        self.getSettings = StorageManager._getSettings(self)
        self.setSettings = StorageManager._setSettings(self)
        self.addImage = StorageManager._addImage(self)
        self.removeImages = StorageManager._removeImages(self)
        self.getMetaData = StorageManager._getMetaData(self)
        self.getImages = StorageManager._getImages(self)
        self.getActivities = StorageManager._getActivities(self)
        self.startActivity = StorageManager._startActivity(self)
        self.startActivityWithFolder = StorageManager._startActivityWithFolder(self)
        self.stopActivity = StorageManager._stopActivity(self)

#
# Section generated by IdlC from "WebcamChannel.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.webcam


# interface
class Channel(Interface):
    idlType = "webcam.Channel:1.0.1"

    NO_ERROR = 0

    ERR_NOT_AVAILABLE = 2

    ERR_NO_SUCH_OBJECT = 3

    class _getClientType(Interface.Method):
        name = 'getClientType'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_string(_ret_, DecodeException)
            return _ret_

    class _getWebcam(Interface.Method):
        name = 'getWebcam'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = Interface.decode(rsp['_ret_'], agent)
            typecheck.is_interface(_ret_, raritan.rpc.webcam.Webcam, DecodeException)
            return _ret_

    class _isAvailable(Interface.Method):
        name = 'isAvailable'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_bool(_ret_, DecodeException)
            return _ret_

    class _release(Interface.Method):
        name = 'release'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            return None

    class _captureImage(Interface.Method):
        name = 'captureImage'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            image = raritan.rpc.webcam.Image.decode(rsp['image'], agent)
            typecheck.is_int(_ret_, DecodeException)
            typecheck.is_struct(image, raritan.rpc.webcam.Image, DecodeException)
            return (_ret_, image)

    class _triggerCapture(Interface.Method):
        name = 'triggerCapture'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            captureToken = rsp['captureToken']
            typecheck.is_int(_ret_, DecodeException)
            typecheck.is_string(captureToken, DecodeException)
            return (_ret_, captureToken)
    def __init__(self, target, agent):
        super(Channel, self).__init__(target, agent)
        self.getClientType = Channel._getClientType(self)
        self.getWebcam = Channel._getWebcam(self)
        self.isAvailable = Channel._isAvailable(self)
        self.release = Channel._release(self)
        self.captureImage = Channel._captureImage(self)
        self.triggerCapture = Channel._triggerCapture(self)

#
# Section generated by IdlC from "WebcamManager.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.idl

import raritan.rpc.webcam


# enumeration
class Priority(Enumeration):
    idlType = "webcam.Priority:1.0.0"
    values = ["VERY_LOW", "LOW", "NORMAL", "HIGH", "VERY_HIGH"]

Priority.VERY_LOW = Priority(0)
Priority.LOW = Priority(1)
Priority.NORMAL = Priority(2)
Priority.HIGH = Priority(3)
Priority.VERY_HIGH = Priority(4)

# value object
class WebcamEvent(raritan.rpc.idl.Event):
    idlType = "webcam.WebcamEvent:1.0.1"

    def __init__(self, webcam, information, name, source):
        super(raritan.rpc.webcam.WebcamEvent, self).__init__(source)
        typecheck.is_interface(webcam, raritan.rpc.webcam.Webcam, AssertionError)
        typecheck.is_struct(information, raritan.rpc.webcam.Information, AssertionError)
        typecheck.is_string(name, AssertionError)

        self.webcam = webcam
        self.information = information
        self.name = name

    def encode(self):
        json = super(raritan.rpc.webcam.WebcamEvent, self).encode()
        json['webcam'] = Interface.encode(self.webcam)
        json['information'] = raritan.rpc.webcam.Information.encode(self.information)
        json['name'] = self.name
        return json

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            webcam = Interface.decode(json['webcam'], agent),
            information = raritan.rpc.webcam.Information.decode(json['information'], agent),
            name = json['name'],
            # for idl.Event
            source = Interface.decode(json['source'], agent),
        )
        return obj

    def listElements(self):
        elements = ["webcam", "information", "name"]
        elements = elements + super(raritan.rpc.webcam.WebcamEvent, self).listElements()
        return elements

# value object
class WebcamAddedEvent(WebcamEvent):
    idlType = "webcam.WebcamAddedEvent:1.0.1"

    def __init__(self, webcam, information, name, source):
        super(raritan.rpc.webcam.WebcamAddedEvent, self).__init__(webcam, information, name, source)

    def encode(self):
        json = super(raritan.rpc.webcam.WebcamAddedEvent, self).encode()
        return json

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            # for webcam.WebcamEvent_1_0_1
            webcam = Interface.decode(json['webcam'], agent),
            information = raritan.rpc.webcam.Information.decode(json['information'], agent),
            name = json['name'],
            # for idl.Event
            source = Interface.decode(json['source'], agent),
        )
        return obj

    def listElements(self):
        elements = []
        elements = elements + super(raritan.rpc.webcam.WebcamAddedEvent, self).listElements()
        return elements

# value object
class WebcamRemovedEvent(WebcamEvent):
    idlType = "webcam.WebcamRemovedEvent:1.0.1"

    def __init__(self, webcam, information, name, source):
        super(raritan.rpc.webcam.WebcamRemovedEvent, self).__init__(webcam, information, name, source)

    def encode(self):
        json = super(raritan.rpc.webcam.WebcamRemovedEvent, self).encode()
        return json

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            # for webcam.WebcamEvent_1_0_1
            webcam = Interface.decode(json['webcam'], agent),
            information = raritan.rpc.webcam.Information.decode(json['information'], agent),
            name = json['name'],
            # for idl.Event
            source = Interface.decode(json['source'], agent),
        )
        return obj

    def listElements(self):
        elements = []
        elements = elements + super(raritan.rpc.webcam.WebcamRemovedEvent, self).listElements()
        return elements

# interface
class WebcamManager(Interface):
    idlType = "webcam.WebcamManager:2.0.1"

    NO_ERROR = 0

    ERR_INVALID_PARAM = 1

    class _getWebcams(Interface.Method):
        name = 'getWebcams'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = [Interface.decode(x0, agent) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_interface(x0, raritan.rpc.webcam.Webcam, DecodeException)
            return _ret_

    class _getChannel(Interface.Method):
        name = 'getChannel'

        @staticmethod
        def encode(webcam, clientType):
            typecheck.is_interface(webcam, raritan.rpc.webcam.Webcam, AssertionError)
            typecheck.is_string(clientType, AssertionError)
            args = {}
            args['webcam'] = Interface.encode(webcam)
            args['clientType'] = clientType
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            channel = Interface.decode(rsp['channel'], agent)
            typecheck.is_int(_ret_, DecodeException)
            typecheck.is_interface(channel, raritan.rpc.webcam.Channel, DecodeException)
            return (_ret_, channel)

    class _getChannels(Interface.Method):
        name = 'getChannels'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = [Interface.decode(x0, agent) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_interface(x0, raritan.rpc.webcam.Channel, DecodeException)
            return _ret_

    class _removeClientType(Interface.Method):
        name = 'removeClientType'

        @staticmethod
        def encode(clientType):
            typecheck.is_string(clientType, AssertionError)
            args = {}
            args['clientType'] = clientType
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _getClientTypes(Interface.Method):
        name = 'getClientTypes'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = [x0 for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_string(x0, DecodeException)
            return _ret_

    class _getClientTypePriorities(Interface.Method):
        name = 'getClientTypePriorities'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = dict([(
                elem['key'],
                raritan.rpc.webcam.Priority.decode(elem['value']))
                for elem in rsp['_ret_']])
            return _ret_

    class _setClientTypePriorities(Interface.Method):
        name = 'setClientTypePriorities'

        @staticmethod
        def encode(priorities):
            args = {}
            args['priorities'] = [dict(
                key = k,
                value = raritan.rpc.webcam.Priority.encode(v))
                for k, v in priorities.items()]
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _getWebcamPriorities(Interface.Method):
        name = 'getWebcamPriorities'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = dict([(
                elem['key'],
                raritan.rpc.webcam.Priority.decode(elem['value']))
                for elem in rsp['_ret_']])
            return _ret_

    class _setWebcamPriorities(Interface.Method):
        name = 'setWebcamPriorities'

        @staticmethod
        def encode(priorities):
            args = {}
            args['priorities'] = [dict(
                key = k,
                value = raritan.rpc.webcam.Priority.encode(v))
                for k, v in priorities.items()]
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(WebcamManager, self).__init__(target, agent)
        self.getWebcams = WebcamManager._getWebcams(self)
        self.getChannel = WebcamManager._getChannel(self)
        self.getChannels = WebcamManager._getChannels(self)
        self.removeClientType = WebcamManager._removeClientType(self)
        self.getClientTypes = WebcamManager._getClientTypes(self)
        self.getClientTypePriorities = WebcamManager._getClientTypePriorities(self)
        self.setClientTypePriorities = WebcamManager._setClientTypePriorities(self)
        self.getWebcamPriorities = WebcamManager._getWebcamPriorities(self)
        self.setWebcamPriorities = WebcamManager._setWebcamPriorities(self)



# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright 2020 Raritan Inc. All rights reserved.
#
# This is an auto-generated file.

#
# Section generated by IdlC from "AssetStripConfig.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.assetmgrmodel

import raritan.rpc.event


# interface
class AssetStripConfig(Interface):
    idlType = "assetmgrmodel.AssetStripConfig:1.0.1"

    # enumeration
    class ScanMode(Enumeration):
        idlType = "assetmgrmodel.AssetStripConfig_1_0_1.ScanMode:1.0.0"
        values = ["SCANMODE_DISABLED", "SCANMODE_BOTH"]

    ScanMode.SCANMODE_DISABLED = ScanMode(0)
    ScanMode.SCANMODE_BOTH = ScanMode(1)

    # enumeration
    class NumberingMode(Enumeration):
        idlType = "assetmgrmodel.AssetStripConfig_1_0_1.NumberingMode:1.0.0"
        values = ["TOP_DOWN", "BOTTOM_UP"]

    NumberingMode.TOP_DOWN = NumberingMode(0)
    NumberingMode.BOTTOM_UP = NumberingMode(1)

    # enumeration
    class Orientation(Enumeration):
        idlType = "assetmgrmodel.AssetStripConfig_1_0_1.Orientation:1.0.0"
        values = ["TOP_CONNECTOR", "BOTTOM_CONNECTOR"]

    Orientation.TOP_CONNECTOR = Orientation(0)
    Orientation.BOTTOM_CONNECTOR = Orientation(1)

    # enumeration
    class LEDOperationMode(Enumeration):
        idlType = "assetmgrmodel.AssetStripConfig_1_0_1.LEDOperationMode:1.0.0"
        values = ["LED_OPERATION_MANUAL", "LED_OPERATION_AUTO"]

    LEDOperationMode.LED_OPERATION_MANUAL = LEDOperationMode(0)
    LEDOperationMode.LED_OPERATION_AUTO = LEDOperationMode(1)

    # enumeration
    class LEDMode(Enumeration):
        idlType = "assetmgrmodel.AssetStripConfig_1_0_1.LEDMode:1.0.0"
        values = ["LED_MODE_ON", "LED_MODE_OFF", "LED_MODE_BLINK_FAST", "LED_MODE_BLINK_SLOW"]

    LEDMode.LED_MODE_ON = LEDMode(0)
    LEDMode.LED_MODE_OFF = LEDMode(1)
    LEDMode.LED_MODE_BLINK_FAST = LEDMode(2)
    LEDMode.LED_MODE_BLINK_SLOW = LEDMode(3)

    # structure
    class LEDColor(Structure):
        idlType = "assetmgrmodel.AssetStripConfig_1_0_1.LEDColor:1.0.0"
        elements = ["r", "g", "b"]

        def __init__(self, r, g, b):
            typecheck.is_int(r, AssertionError)
            typecheck.is_int(g, AssertionError)
            typecheck.is_int(b, AssertionError)

            self.r = r
            self.g = g
            self.b = b

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                r = json['r'],
                g = json['g'],
                b = json['b'],
            )
            return obj

        def encode(self):
            json = {}
            json['r'] = self.r
            json['g'] = self.g
            json['b'] = self.b
            return json

    # structure
    class StripSettings(Structure):
        idlType = "assetmgrmodel.AssetStripConfig_1_0_1.StripSettings:1.0.0"
        elements = ["rackUnitCount", "name", "scanMode", "defaultColorConnected", "defaultColorDisconnected", "numberingMode", "numberingOffset", "orientation"]

        def __init__(self, rackUnitCount, name, scanMode, defaultColorConnected, defaultColorDisconnected, numberingMode, numberingOffset, orientation):
            typecheck.is_int(rackUnitCount, AssertionError)
            typecheck.is_string(name, AssertionError)
            typecheck.is_enum(scanMode, raritan.rpc.assetmgrmodel.AssetStripConfig.ScanMode, AssertionError)
            typecheck.is_struct(defaultColorConnected, raritan.rpc.assetmgrmodel.AssetStripConfig.LEDColor, AssertionError)
            typecheck.is_struct(defaultColorDisconnected, raritan.rpc.assetmgrmodel.AssetStripConfig.LEDColor, AssertionError)
            typecheck.is_enum(numberingMode, raritan.rpc.assetmgrmodel.AssetStripConfig.NumberingMode, AssertionError)
            typecheck.is_int(numberingOffset, AssertionError)
            typecheck.is_enum(orientation, raritan.rpc.assetmgrmodel.AssetStripConfig.Orientation, AssertionError)

            self.rackUnitCount = rackUnitCount
            self.name = name
            self.scanMode = scanMode
            self.defaultColorConnected = defaultColorConnected
            self.defaultColorDisconnected = defaultColorDisconnected
            self.numberingMode = numberingMode
            self.numberingOffset = numberingOffset
            self.orientation = orientation

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                rackUnitCount = json['rackUnitCount'],
                name = json['name'],
                scanMode = raritan.rpc.assetmgrmodel.AssetStripConfig.ScanMode.decode(json['scanMode']),
                defaultColorConnected = raritan.rpc.assetmgrmodel.AssetStripConfig.LEDColor.decode(json['defaultColorConnected'], agent),
                defaultColorDisconnected = raritan.rpc.assetmgrmodel.AssetStripConfig.LEDColor.decode(json['defaultColorDisconnected'], agent),
                numberingMode = raritan.rpc.assetmgrmodel.AssetStripConfig.NumberingMode.decode(json['numberingMode']),
                numberingOffset = json['numberingOffset'],
                orientation = raritan.rpc.assetmgrmodel.AssetStripConfig.Orientation.decode(json['orientation']),
            )
            return obj

        def encode(self):
            json = {}
            json['rackUnitCount'] = self.rackUnitCount
            json['name'] = self.name
            json['scanMode'] = raritan.rpc.assetmgrmodel.AssetStripConfig.ScanMode.encode(self.scanMode)
            json['defaultColorConnected'] = raritan.rpc.assetmgrmodel.AssetStripConfig.LEDColor.encode(self.defaultColorConnected)
            json['defaultColorDisconnected'] = raritan.rpc.assetmgrmodel.AssetStripConfig.LEDColor.encode(self.defaultColorDisconnected)
            json['numberingMode'] = raritan.rpc.assetmgrmodel.AssetStripConfig.NumberingMode.encode(self.numberingMode)
            json['numberingOffset'] = self.numberingOffset
            json['orientation'] = raritan.rpc.assetmgrmodel.AssetStripConfig.Orientation.encode(self.orientation)
            return json

    # structure
    class RackUnitSettings(Structure):
        idlType = "assetmgrmodel.AssetStripConfig_1_0_1.RackUnitSettings:1.0.0"
        elements = ["opmode", "mode", "color", "name"]

        def __init__(self, opmode, mode, color, name):
            typecheck.is_enum(opmode, raritan.rpc.assetmgrmodel.AssetStripConfig.LEDOperationMode, AssertionError)
            typecheck.is_enum(mode, raritan.rpc.assetmgrmodel.AssetStripConfig.LEDMode, AssertionError)
            typecheck.is_struct(color, raritan.rpc.assetmgrmodel.AssetStripConfig.LEDColor, AssertionError)
            typecheck.is_string(name, AssertionError)

            self.opmode = opmode
            self.mode = mode
            self.color = color
            self.name = name

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                opmode = raritan.rpc.assetmgrmodel.AssetStripConfig.LEDOperationMode.decode(json['opmode']),
                mode = raritan.rpc.assetmgrmodel.AssetStripConfig.LEDMode.decode(json['mode']),
                color = raritan.rpc.assetmgrmodel.AssetStripConfig.LEDColor.decode(json['color'], agent),
                name = json['name'],
            )
            return obj

        def encode(self):
            json = {}
            json['opmode'] = raritan.rpc.assetmgrmodel.AssetStripConfig.LEDOperationMode.encode(self.opmode)
            json['mode'] = raritan.rpc.assetmgrmodel.AssetStripConfig.LEDMode.encode(self.mode)
            json['color'] = raritan.rpc.assetmgrmodel.AssetStripConfig.LEDColor.encode(self.color)
            json['name'] = self.name
            return json

    # value object
    class StripSettingsChangedEvent(raritan.rpc.event.UserEvent):
        idlType = "assetmgrmodel.AssetStripConfig_1_0_1.StripSettingsChangedEvent:1.0.0"

        def __init__(self, oldSettings, newSettings, actUserName, actIpAddr, source):
            super(raritan.rpc.assetmgrmodel.AssetStripConfig.StripSettingsChangedEvent, self).__init__(actUserName, actIpAddr, source)
            typecheck.is_struct(oldSettings, raritan.rpc.assetmgrmodel.AssetStripConfig.StripSettings, AssertionError)
            typecheck.is_struct(newSettings, raritan.rpc.assetmgrmodel.AssetStripConfig.StripSettings, AssertionError)

            self.oldSettings = oldSettings
            self.newSettings = newSettings

        def encode(self):
            json = super(raritan.rpc.assetmgrmodel.AssetStripConfig.StripSettingsChangedEvent, self).encode()
            json['oldSettings'] = raritan.rpc.assetmgrmodel.AssetStripConfig.StripSettings.encode(self.oldSettings)
            json['newSettings'] = raritan.rpc.assetmgrmodel.AssetStripConfig.StripSettings.encode(self.newSettings)
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                oldSettings = raritan.rpc.assetmgrmodel.AssetStripConfig.StripSettings.decode(json['oldSettings'], agent),
                newSettings = raritan.rpc.assetmgrmodel.AssetStripConfig.StripSettings.decode(json['newSettings'], agent),
                # for event.UserEvent
                actUserName = json['actUserName'],
                actIpAddr = json['actIpAddr'],
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["oldSettings", "newSettings"]
            elements = elements + super(raritan.rpc.assetmgrmodel.AssetStripConfig.StripSettingsChangedEvent, self).listElements()
            return elements

    # value object
    class RackUnitSettingsChangedEvent(raritan.rpc.event.UserEvent):
        idlType = "assetmgrmodel.AssetStripConfig_1_0_1.RackUnitSettingsChangedEvent:1.0.0"

        def __init__(self, rackUnitNumber, oldSettings, newSettings, actUserName, actIpAddr, source):
            super(raritan.rpc.assetmgrmodel.AssetStripConfig.RackUnitSettingsChangedEvent, self).__init__(actUserName, actIpAddr, source)
            typecheck.is_int(rackUnitNumber, AssertionError)
            typecheck.is_struct(oldSettings, raritan.rpc.assetmgrmodel.AssetStripConfig.RackUnitSettings, AssertionError)
            typecheck.is_struct(newSettings, raritan.rpc.assetmgrmodel.AssetStripConfig.RackUnitSettings, AssertionError)

            self.rackUnitNumber = rackUnitNumber
            self.oldSettings = oldSettings
            self.newSettings = newSettings

        def encode(self):
            json = super(raritan.rpc.assetmgrmodel.AssetStripConfig.RackUnitSettingsChangedEvent, self).encode()
            json['rackUnitNumber'] = self.rackUnitNumber
            json['oldSettings'] = raritan.rpc.assetmgrmodel.AssetStripConfig.RackUnitSettings.encode(self.oldSettings)
            json['newSettings'] = raritan.rpc.assetmgrmodel.AssetStripConfig.RackUnitSettings.encode(self.newSettings)
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                rackUnitNumber = json['rackUnitNumber'],
                oldSettings = raritan.rpc.assetmgrmodel.AssetStripConfig.RackUnitSettings.decode(json['oldSettings'], agent),
                newSettings = raritan.rpc.assetmgrmodel.AssetStripConfig.RackUnitSettings.decode(json['newSettings'], agent),
                # for event.UserEvent
                actUserName = json['actUserName'],
                actIpAddr = json['actIpAddr'],
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["rackUnitNumber", "oldSettings", "newSettings"]
            elements = elements + super(raritan.rpc.assetmgrmodel.AssetStripConfig.RackUnitSettingsChangedEvent, self).listElements()
            return elements

    class _getStripSettings(Interface.Method):
        name = 'getStripSettings'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.assetmgrmodel.AssetStripConfig.StripSettings.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.assetmgrmodel.AssetStripConfig.StripSettings, DecodeException)
            return _ret_

    class _setStripSettings(Interface.Method):
        name = 'setStripSettings'

        @staticmethod
        def encode(settings):
            typecheck.is_struct(settings, raritan.rpc.assetmgrmodel.AssetStripConfig.StripSettings, AssertionError)
            args = {}
            args['settings'] = raritan.rpc.assetmgrmodel.AssetStripConfig.StripSettings.encode(settings)
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _getRackUnitSettings(Interface.Method):
        name = 'getRackUnitSettings'

        @staticmethod
        def encode(rackUnitNumber):
            typecheck.is_int(rackUnitNumber, AssertionError)
            args = {}
            args['rackUnitNumber'] = rackUnitNumber
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            settings = raritan.rpc.assetmgrmodel.AssetStripConfig.RackUnitSettings.decode(rsp['settings'], agent)
            typecheck.is_int(_ret_, DecodeException)
            typecheck.is_struct(settings, raritan.rpc.assetmgrmodel.AssetStripConfig.RackUnitSettings, DecodeException)
            return (_ret_, settings)

    class _getAllRackUnitSettings(Interface.Method):
        name = 'getAllRackUnitSettings'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = [raritan.rpc.assetmgrmodel.AssetStripConfig.RackUnitSettings.decode(x0, agent) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_struct(x0, raritan.rpc.assetmgrmodel.AssetStripConfig.RackUnitSettings, DecodeException)
            return _ret_

    class _setRackUnitSettings(Interface.Method):
        name = 'setRackUnitSettings'

        @staticmethod
        def encode(rackUnitNumber, settings):
            typecheck.is_int(rackUnitNumber, AssertionError)
            typecheck.is_struct(settings, raritan.rpc.assetmgrmodel.AssetStripConfig.RackUnitSettings, AssertionError)
            args = {}
            args['rackUnitNumber'] = rackUnitNumber
            args['settings'] = raritan.rpc.assetmgrmodel.AssetStripConfig.RackUnitSettings.encode(settings)
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _setMultipleRackUnitSettings(Interface.Method):
        name = 'setMultipleRackUnitSettings'

        @staticmethod
        def encode(settings):
            args = {}
            args['settings'] = [dict(
                key = k,
                value = raritan.rpc.assetmgrmodel.AssetStripConfig.RackUnitSettings.encode(v))
                for k, v in settings.items()]
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(AssetStripConfig, self).__init__(target, agent)
        self.getStripSettings = AssetStripConfig._getStripSettings(self)
        self.setStripSettings = AssetStripConfig._setStripSettings(self)
        self.getRackUnitSettings = AssetStripConfig._getRackUnitSettings(self)
        self.getAllRackUnitSettings = AssetStripConfig._getAllRackUnitSettings(self)
        self.setRackUnitSettings = AssetStripConfig._setRackUnitSettings(self)
        self.setMultipleRackUnitSettings = AssetStripConfig._setMultipleRackUnitSettings(self)

#
# Section generated by IdlC from "AssetStrip.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.assetmgrmodel

import raritan.rpc.event

import raritan.rpc.idl


# interface
class AssetStrip(Interface):
    idlType = "assetmgrmodel.AssetStrip:2.0.5"

    NO_ERROR = 0

    ERR_INVALID_PARAM = 1

    ERR_NO_SUCH_OBJECT = 2

    ERR_NOT_SUPPORTED = 3

    ERR_OPERATION_FAILED = 4

    MAIN_STRIP_COLUMN = 0

    # enumeration
    class State(Enumeration):
        idlType = "assetmgrmodel.AssetStrip_2_0_5.State:1.0.0"
        values = ["DISCONNECTED", "FIRMWARE_UPDATE", "UNSUPPORTED", "AVAILABLE"]

    State.DISCONNECTED = State(0)
    State.FIRMWARE_UPDATE = State(1)
    State.UNSUPPORTED = State(2)
    State.AVAILABLE = State(3)

    # enumeration
    class StripType(Enumeration):
        idlType = "assetmgrmodel.AssetStrip_2_0_5.StripType:1.0.0"
        values = ["SIMPLE", "COMPOSITE"]

    StripType.SIMPLE = StripType(0)
    StripType.COMPOSITE = StripType(1)

    # enumeration
    class TagType(Enumeration):
        idlType = "assetmgrmodel.AssetStrip_2_0_5.TagType:1.0.0"
        values = ["SINGLE", "EXTENSION", "NONE"]

    TagType.SINGLE = TagType(0)
    TagType.EXTENSION = TagType(1)
    TagType.NONE = TagType(2)

    # enumeration
    class CascadeState(Enumeration):
        idlType = "assetmgrmodel.AssetStrip_2_0_5.CascadeState:1.0.0"
        values = ["CASCADE_ACTIVE", "CASCADE_FIRMWARE_UPDATE"]

    CascadeState.CASCADE_ACTIVE = CascadeState(0)
    CascadeState.CASCADE_FIRMWARE_UPDATE = CascadeState(1)

    # structure
    class DeviceInfo(Structure):
        idlType = "assetmgrmodel.AssetStrip_2_0_5.DeviceInfo:1.0.0"
        elements = ["deviceId", "hardwareId", "protocolVersion", "bootVersion", "appVersion", "orientationSensAvailable", "isCascadable", "rackUnitCountConfigurable"]

        def __init__(self, deviceId, hardwareId, protocolVersion, bootVersion, appVersion, orientationSensAvailable, isCascadable, rackUnitCountConfigurable):
            typecheck.is_int(deviceId, AssertionError)
            typecheck.is_int(hardwareId, AssertionError)
            typecheck.is_int(protocolVersion, AssertionError)
            typecheck.is_int(bootVersion, AssertionError)
            typecheck.is_int(appVersion, AssertionError)
            typecheck.is_bool(orientationSensAvailable, AssertionError)
            typecheck.is_bool(isCascadable, AssertionError)
            typecheck.is_bool(rackUnitCountConfigurable, AssertionError)

            self.deviceId = deviceId
            self.hardwareId = hardwareId
            self.protocolVersion = protocolVersion
            self.bootVersion = bootVersion
            self.appVersion = appVersion
            self.orientationSensAvailable = orientationSensAvailable
            self.isCascadable = isCascadable
            self.rackUnitCountConfigurable = rackUnitCountConfigurable

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                deviceId = json['deviceId'],
                hardwareId = json['hardwareId'],
                protocolVersion = json['protocolVersion'],
                bootVersion = json['bootVersion'],
                appVersion = json['appVersion'],
                orientationSensAvailable = json['orientationSensAvailable'],
                isCascadable = json['isCascadable'],
                rackUnitCountConfigurable = json['rackUnitCountConfigurable'],
            )
            return obj

        def encode(self):
            json = {}
            json['deviceId'] = self.deviceId
            json['hardwareId'] = self.hardwareId
            json['protocolVersion'] = self.protocolVersion
            json['bootVersion'] = self.bootVersion
            json['appVersion'] = self.appVersion
            json['orientationSensAvailable'] = self.orientationSensAvailable
            json['isCascadable'] = self.isCascadable
            json['rackUnitCountConfigurable'] = self.rackUnitCountConfigurable
            return json

    # structure
    class StripInfo(Structure):
        idlType = "assetmgrmodel.AssetStrip_2_0_5.StripInfo:1.0.0"
        elements = ["maxMainTagCount", "maxBladeTagCount", "mainTagCount", "bladeTagCount", "bladeOverflow", "rackUnitCount", "componentCount", "cascadeState"]

        def __init__(self, maxMainTagCount, maxBladeTagCount, mainTagCount, bladeTagCount, bladeOverflow, rackUnitCount, componentCount, cascadeState):
            typecheck.is_int(maxMainTagCount, AssertionError)
            typecheck.is_int(maxBladeTagCount, AssertionError)
            typecheck.is_int(mainTagCount, AssertionError)
            typecheck.is_int(bladeTagCount, AssertionError)
            typecheck.is_bool(bladeOverflow, AssertionError)
            typecheck.is_int(rackUnitCount, AssertionError)
            typecheck.is_int(componentCount, AssertionError)
            typecheck.is_enum(cascadeState, raritan.rpc.assetmgrmodel.AssetStrip.CascadeState, AssertionError)

            self.maxMainTagCount = maxMainTagCount
            self.maxBladeTagCount = maxBladeTagCount
            self.mainTagCount = mainTagCount
            self.bladeTagCount = bladeTagCount
            self.bladeOverflow = bladeOverflow
            self.rackUnitCount = rackUnitCount
            self.componentCount = componentCount
            self.cascadeState = cascadeState

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                maxMainTagCount = json['maxMainTagCount'],
                maxBladeTagCount = json['maxBladeTagCount'],
                mainTagCount = json['mainTagCount'],
                bladeTagCount = json['bladeTagCount'],
                bladeOverflow = json['bladeOverflow'],
                rackUnitCount = json['rackUnitCount'],
                componentCount = json['componentCount'],
                cascadeState = raritan.rpc.assetmgrmodel.AssetStrip.CascadeState.decode(json['cascadeState']),
            )
            return obj

        def encode(self):
            json = {}
            json['maxMainTagCount'] = self.maxMainTagCount
            json['maxBladeTagCount'] = self.maxBladeTagCount
            json['mainTagCount'] = self.mainTagCount
            json['bladeTagCount'] = self.bladeTagCount
            json['bladeOverflow'] = self.bladeOverflow
            json['rackUnitCount'] = self.rackUnitCount
            json['componentCount'] = self.componentCount
            json['cascadeState'] = raritan.rpc.assetmgrmodel.AssetStrip.CascadeState.encode(self.cascadeState)
            return json

    # structure
    class TagInfo(Structure):
        idlType = "assetmgrmodel.AssetStrip_2_0_5.TagInfo:1.0.0"
        elements = ["rackUnitNumber", "slotNumber", "familyDesc", "rawId", "programmable"]

        def __init__(self, rackUnitNumber, slotNumber, familyDesc, rawId, programmable):
            typecheck.is_int(rackUnitNumber, AssertionError)
            typecheck.is_int(slotNumber, AssertionError)
            typecheck.is_string(familyDesc, AssertionError)
            typecheck.is_string(rawId, AssertionError)
            typecheck.is_bool(programmable, AssertionError)

            self.rackUnitNumber = rackUnitNumber
            self.slotNumber = slotNumber
            self.familyDesc = familyDesc
            self.rawId = rawId
            self.programmable = programmable

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                rackUnitNumber = json['rackUnitNumber'],
                slotNumber = json['slotNumber'],
                familyDesc = json['familyDesc'],
                rawId = json['rawId'],
                programmable = json['programmable'],
            )
            return obj

        def encode(self):
            json = {}
            json['rackUnitNumber'] = self.rackUnitNumber
            json['slotNumber'] = self.slotNumber
            json['familyDesc'] = self.familyDesc
            json['rawId'] = self.rawId
            json['programmable'] = self.programmable
            return json

    # structure
    class RackUnitInfo(Structure):
        idlType = "assetmgrmodel.AssetStrip_2_0_5.RackUnitInfo:1.0.0"
        elements = ["rackUnitNumber", "rackUnitPosition", "type", "size", "settings", "assetStripCascadePosition", "rackUnitRelativePosition", "assetStripNumberOfRackUnits"]

        def __init__(self, rackUnitNumber, rackUnitPosition, type, size, settings, assetStripCascadePosition, rackUnitRelativePosition, assetStripNumberOfRackUnits):
            typecheck.is_int(rackUnitNumber, AssertionError)
            typecheck.is_int(rackUnitPosition, AssertionError)
            typecheck.is_enum(type, raritan.rpc.assetmgrmodel.AssetStrip.TagType, AssertionError)
            typecheck.is_int(size, AssertionError)
            typecheck.is_struct(settings, raritan.rpc.assetmgrmodel.AssetStripConfig.RackUnitSettings, AssertionError)
            typecheck.is_int(assetStripCascadePosition, AssertionError)
            typecheck.is_int(rackUnitRelativePosition, AssertionError)
            typecheck.is_int(assetStripNumberOfRackUnits, AssertionError)

            self.rackUnitNumber = rackUnitNumber
            self.rackUnitPosition = rackUnitPosition
            self.type = type
            self.size = size
            self.settings = settings
            self.assetStripCascadePosition = assetStripCascadePosition
            self.rackUnitRelativePosition = rackUnitRelativePosition
            self.assetStripNumberOfRackUnits = assetStripNumberOfRackUnits

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                rackUnitNumber = json['rackUnitNumber'],
                rackUnitPosition = json['rackUnitPosition'],
                type = raritan.rpc.assetmgrmodel.AssetStrip.TagType.decode(json['type']),
                size = json['size'],
                settings = raritan.rpc.assetmgrmodel.AssetStripConfig.RackUnitSettings.decode(json['settings'], agent),
                assetStripCascadePosition = json['assetStripCascadePosition'],
                rackUnitRelativePosition = json['rackUnitRelativePosition'],
                assetStripNumberOfRackUnits = json['assetStripNumberOfRackUnits'],
            )
            return obj

        def encode(self):
            json = {}
            json['rackUnitNumber'] = self.rackUnitNumber
            json['rackUnitPosition'] = self.rackUnitPosition
            json['type'] = raritan.rpc.assetmgrmodel.AssetStrip.TagType.encode(self.type)
            json['size'] = self.size
            json['settings'] = raritan.rpc.assetmgrmodel.AssetStripConfig.RackUnitSettings.encode(self.settings)
            json['assetStripCascadePosition'] = self.assetStripCascadePosition
            json['rackUnitRelativePosition'] = self.rackUnitRelativePosition
            json['assetStripNumberOfRackUnits'] = self.assetStripNumberOfRackUnits
            return json

    # value object
    class StripInfoChangedEvent(raritan.rpc.idl.Event):
        idlType = "assetmgrmodel.AssetStrip_2_0_5.StripInfoChangedEvent:1.0.0"

        def __init__(self, oldInfo, newInfo, source):
            super(raritan.rpc.assetmgrmodel.AssetStrip.StripInfoChangedEvent, self).__init__(source)
            typecheck.is_struct(oldInfo, raritan.rpc.assetmgrmodel.AssetStrip.StripInfo, AssertionError)
            typecheck.is_struct(newInfo, raritan.rpc.assetmgrmodel.AssetStrip.StripInfo, AssertionError)

            self.oldInfo = oldInfo
            self.newInfo = newInfo

        def encode(self):
            json = super(raritan.rpc.assetmgrmodel.AssetStrip.StripInfoChangedEvent, self).encode()
            json['oldInfo'] = raritan.rpc.assetmgrmodel.AssetStrip.StripInfo.encode(self.oldInfo)
            json['newInfo'] = raritan.rpc.assetmgrmodel.AssetStrip.StripInfo.encode(self.newInfo)
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                oldInfo = raritan.rpc.assetmgrmodel.AssetStrip.StripInfo.decode(json['oldInfo'], agent),
                newInfo = raritan.rpc.assetmgrmodel.AssetStrip.StripInfo.decode(json['newInfo'], agent),
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["oldInfo", "newInfo"]
            elements = elements + super(raritan.rpc.assetmgrmodel.AssetStrip.StripInfoChangedEvent, self).listElements()
            return elements

    # value object
    class StateChangedEvent(raritan.rpc.idl.Event):
        idlType = "assetmgrmodel.AssetStrip_2_0_5.StateChangedEvent:1.0.0"

        def __init__(self, oldState, newState, deviceInfo, source):
            super(raritan.rpc.assetmgrmodel.AssetStrip.StateChangedEvent, self).__init__(source)
            typecheck.is_enum(oldState, raritan.rpc.assetmgrmodel.AssetStrip.State, AssertionError)
            typecheck.is_enum(newState, raritan.rpc.assetmgrmodel.AssetStrip.State, AssertionError)
            typecheck.is_struct(deviceInfo, raritan.rpc.assetmgrmodel.AssetStrip.DeviceInfo, AssertionError)

            self.oldState = oldState
            self.newState = newState
            self.deviceInfo = deviceInfo

        def encode(self):
            json = super(raritan.rpc.assetmgrmodel.AssetStrip.StateChangedEvent, self).encode()
            json['oldState'] = raritan.rpc.assetmgrmodel.AssetStrip.State.encode(self.oldState)
            json['newState'] = raritan.rpc.assetmgrmodel.AssetStrip.State.encode(self.newState)
            json['deviceInfo'] = raritan.rpc.assetmgrmodel.AssetStrip.DeviceInfo.encode(self.deviceInfo)
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                oldState = raritan.rpc.assetmgrmodel.AssetStrip.State.decode(json['oldState']),
                newState = raritan.rpc.assetmgrmodel.AssetStrip.State.decode(json['newState']),
                deviceInfo = raritan.rpc.assetmgrmodel.AssetStrip.DeviceInfo.decode(json['deviceInfo'], agent),
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["oldState", "newState", "deviceInfo"]
            elements = elements + super(raritan.rpc.assetmgrmodel.AssetStrip.StateChangedEvent, self).listElements()
            return elements

    # value object
    class RackUnitChangedEvent(raritan.rpc.event.UserEvent):
        idlType = "assetmgrmodel.AssetStrip_2_0_5.RackUnitChangedEvent:1.0.0"

        def __init__(self, rackUnitNumber, rackUnit, actUserName, actIpAddr, source):
            super(raritan.rpc.assetmgrmodel.AssetStrip.RackUnitChangedEvent, self).__init__(actUserName, actIpAddr, source)
            typecheck.is_int(rackUnitNumber, AssertionError)
            typecheck.is_struct(rackUnit, raritan.rpc.assetmgrmodel.AssetStrip.RackUnitInfo, AssertionError)

            self.rackUnitNumber = rackUnitNumber
            self.rackUnit = rackUnit

        def encode(self):
            json = super(raritan.rpc.assetmgrmodel.AssetStrip.RackUnitChangedEvent, self).encode()
            json['rackUnitNumber'] = self.rackUnitNumber
            json['rackUnit'] = raritan.rpc.assetmgrmodel.AssetStrip.RackUnitInfo.encode(self.rackUnit)
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                rackUnitNumber = json['rackUnitNumber'],
                rackUnit = raritan.rpc.assetmgrmodel.AssetStrip.RackUnitInfo.decode(json['rackUnit'], agent),
                # for event.UserEvent
                actUserName = json['actUserName'],
                actIpAddr = json['actIpAddr'],
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["rackUnitNumber", "rackUnit"]
            elements = elements + super(raritan.rpc.assetmgrmodel.AssetStrip.RackUnitChangedEvent, self).listElements()
            return elements

    # structure
    class TagChangeInfo(Structure):
        idlType = "assetmgrmodel.AssetStrip_2_0_5.TagChangeInfo:1.0.0"
        elements = ["tag", "info", "parentBladeTagId", "slotPosition"]

        def __init__(self, tag, info, parentBladeTagId, slotPosition):
            typecheck.is_struct(tag, raritan.rpc.assetmgrmodel.AssetStrip.TagInfo, AssertionError)
            typecheck.is_struct(info, raritan.rpc.assetmgrmodel.AssetStrip.RackUnitInfo, AssertionError)
            typecheck.is_string(parentBladeTagId, AssertionError)
            typecheck.is_int(slotPosition, AssertionError)

            self.tag = tag
            self.info = info
            self.parentBladeTagId = parentBladeTagId
            self.slotPosition = slotPosition

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                tag = raritan.rpc.assetmgrmodel.AssetStrip.TagInfo.decode(json['tag'], agent),
                info = raritan.rpc.assetmgrmodel.AssetStrip.RackUnitInfo.decode(json['info'], agent),
                parentBladeTagId = json['parentBladeTagId'],
                slotPosition = json['slotPosition'],
            )
            return obj

        def encode(self):
            json = {}
            json['tag'] = raritan.rpc.assetmgrmodel.AssetStrip.TagInfo.encode(self.tag)
            json['info'] = raritan.rpc.assetmgrmodel.AssetStrip.RackUnitInfo.encode(self.info)
            json['parentBladeTagId'] = self.parentBladeTagId
            json['slotPosition'] = self.slotPosition
            return json

    # value object
    class TagEvent(raritan.rpc.idl.Event):
        idlType = "assetmgrmodel.AssetStrip_2_0_5.TagEvent:1.0.0"

        def __init__(self, tags, allTags, source):
            super(raritan.rpc.assetmgrmodel.AssetStrip.TagEvent, self).__init__(source)
            for x0 in tags:
                typecheck.is_struct(x0, raritan.rpc.assetmgrmodel.AssetStrip.TagChangeInfo, AssertionError)
            for x0 in allTags:
                typecheck.is_struct(x0, raritan.rpc.assetmgrmodel.AssetStrip.TagInfo, AssertionError)

            self.tags = tags
            self.allTags = allTags

        def encode(self):
            json = super(raritan.rpc.assetmgrmodel.AssetStrip.TagEvent, self).encode()
            json['tags'] = [raritan.rpc.assetmgrmodel.AssetStrip.TagChangeInfo.encode(x0) for x0 in self.tags]
            json['allTags'] = [raritan.rpc.assetmgrmodel.AssetStrip.TagInfo.encode(x0) for x0 in self.allTags]
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                tags = [raritan.rpc.assetmgrmodel.AssetStrip.TagChangeInfo.decode(x0, agent) for x0 in json['tags']],
                allTags = [raritan.rpc.assetmgrmodel.AssetStrip.TagInfo.decode(x0, agent) for x0 in json['allTags']],
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["tags", "allTags"]
            elements = elements + super(raritan.rpc.assetmgrmodel.AssetStrip.TagEvent, self).listElements()
            return elements

    # value object
    class TagAddedEvent(TagEvent):
        idlType = "assetmgrmodel.AssetStrip_2_0_5.TagAddedEvent:1.0.0"

        def __init__(self, tags, allTags, source):
            super(raritan.rpc.assetmgrmodel.AssetStrip.TagAddedEvent, self).__init__(tags, allTags, source)

        def encode(self):
            json = super(raritan.rpc.assetmgrmodel.AssetStrip.TagAddedEvent, self).encode()
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                # for assetmgrmodel.AssetStrip_2_0_5.TagEvent
                tags = [raritan.rpc.assetmgrmodel.AssetStrip.TagChangeInfo.decode(x0, agent) for x0 in json['tags']],
                allTags = [raritan.rpc.assetmgrmodel.AssetStrip.TagInfo.decode(x0, agent) for x0 in json['allTags']],
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = []
            elements = elements + super(raritan.rpc.assetmgrmodel.AssetStrip.TagAddedEvent, self).listElements()
            return elements

    # value object
    class TagRemovedEvent(TagEvent):
        idlType = "assetmgrmodel.AssetStrip_2_0_5.TagRemovedEvent:1.0.0"

        def __init__(self, tags, allTags, source):
            super(raritan.rpc.assetmgrmodel.AssetStrip.TagRemovedEvent, self).__init__(tags, allTags, source)

        def encode(self):
            json = super(raritan.rpc.assetmgrmodel.AssetStrip.TagRemovedEvent, self).encode()
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                # for assetmgrmodel.AssetStrip_2_0_5.TagEvent
                tags = [raritan.rpc.assetmgrmodel.AssetStrip.TagChangeInfo.decode(x0, agent) for x0 in json['tags']],
                allTags = [raritan.rpc.assetmgrmodel.AssetStrip.TagInfo.decode(x0, agent) for x0 in json['allTags']],
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = []
            elements = elements + super(raritan.rpc.assetmgrmodel.AssetStrip.TagRemovedEvent, self).listElements()
            return elements

    # enumeration
    class FirmwareUpdateState(Enumeration):
        idlType = "assetmgrmodel.AssetStrip_2_0_5.FirmwareUpdateState:1.0.0"
        values = ["UPDATE_STARTED", "UPDATE_SUCCESSFUL", "UPDATE_FAILED"]

    FirmwareUpdateState.UPDATE_STARTED = FirmwareUpdateState(0)
    FirmwareUpdateState.UPDATE_SUCCESSFUL = FirmwareUpdateState(1)
    FirmwareUpdateState.UPDATE_FAILED = FirmwareUpdateState(2)

    # value object
    class FirmwareUpdateStateChangedEvent(raritan.rpc.idl.Event):
        idlType = "assetmgrmodel.AssetStrip_2_0_5.FirmwareUpdateStateChangedEvent:1.0.0"

        def __init__(self, state, source):
            super(raritan.rpc.assetmgrmodel.AssetStrip.FirmwareUpdateStateChangedEvent, self).__init__(source)
            typecheck.is_enum(state, raritan.rpc.assetmgrmodel.AssetStrip.FirmwareUpdateState, AssertionError)

            self.state = state

        def encode(self):
            json = super(raritan.rpc.assetmgrmodel.AssetStrip.FirmwareUpdateStateChangedEvent, self).encode()
            json['state'] = raritan.rpc.assetmgrmodel.AssetStrip.FirmwareUpdateState.encode(self.state)
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                state = raritan.rpc.assetmgrmodel.AssetStrip.FirmwareUpdateState.decode(json['state']),
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["state"]
            elements = elements + super(raritan.rpc.assetmgrmodel.AssetStrip.FirmwareUpdateStateChangedEvent, self).listElements()
            return elements

    # value object
    class BladeOverflowChangedEvent(raritan.rpc.idl.Event):
        idlType = "assetmgrmodel.AssetStrip_2_0_5.BladeOverflowChangedEvent:1.0.0"

        def __init__(self, overflow, source):
            super(raritan.rpc.assetmgrmodel.AssetStrip.BladeOverflowChangedEvent, self).__init__(source)
            typecheck.is_bool(overflow, AssertionError)

            self.overflow = overflow

        def encode(self):
            json = super(raritan.rpc.assetmgrmodel.AssetStrip.BladeOverflowChangedEvent, self).encode()
            json['overflow'] = self.overflow
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                overflow = json['overflow'],
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["overflow"]
            elements = elements + super(raritan.rpc.assetmgrmodel.AssetStrip.BladeOverflowChangedEvent, self).listElements()
            return elements

    # value object
    class OrientationChangedEvent(raritan.rpc.idl.Event):
        idlType = "assetmgrmodel.AssetStrip_2_0_5.OrientationChangedEvent:1.0.0"

        def __init__(self, oldOrientation, newOrientation, source):
            super(raritan.rpc.assetmgrmodel.AssetStrip.OrientationChangedEvent, self).__init__(source)
            typecheck.is_enum(oldOrientation, raritan.rpc.assetmgrmodel.AssetStripConfig.Orientation, AssertionError)
            typecheck.is_enum(newOrientation, raritan.rpc.assetmgrmodel.AssetStripConfig.Orientation, AssertionError)

            self.oldOrientation = oldOrientation
            self.newOrientation = newOrientation

        def encode(self):
            json = super(raritan.rpc.assetmgrmodel.AssetStrip.OrientationChangedEvent, self).encode()
            json['oldOrientation'] = raritan.rpc.assetmgrmodel.AssetStripConfig.Orientation.encode(self.oldOrientation)
            json['newOrientation'] = raritan.rpc.assetmgrmodel.AssetStripConfig.Orientation.encode(self.newOrientation)
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                oldOrientation = raritan.rpc.assetmgrmodel.AssetStripConfig.Orientation.decode(json['oldOrientation']),
                newOrientation = raritan.rpc.assetmgrmodel.AssetStripConfig.Orientation.decode(json['newOrientation']),
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["oldOrientation", "newOrientation"]
            elements = elements + super(raritan.rpc.assetmgrmodel.AssetStrip.OrientationChangedEvent, self).listElements()
            return elements

    # value object
    class CompositionChangedEvent(raritan.rpc.idl.Event):
        idlType = "assetmgrmodel.AssetStrip_2_0_5.CompositionChangedEvent:1.0.0"

        def __init__(self, oldComponentCount, newComponentCount, source):
            super(raritan.rpc.assetmgrmodel.AssetStrip.CompositionChangedEvent, self).__init__(source)
            typecheck.is_int(oldComponentCount, AssertionError)
            typecheck.is_int(newComponentCount, AssertionError)

            self.oldComponentCount = oldComponentCount
            self.newComponentCount = newComponentCount

        def encode(self):
            json = super(raritan.rpc.assetmgrmodel.AssetStrip.CompositionChangedEvent, self).encode()
            json['oldComponentCount'] = self.oldComponentCount
            json['newComponentCount'] = self.newComponentCount
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                oldComponentCount = json['oldComponentCount'],
                newComponentCount = json['newComponentCount'],
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["oldComponentCount", "newComponentCount"]
            elements = elements + super(raritan.rpc.assetmgrmodel.AssetStrip.CompositionChangedEvent, self).listElements()
            return elements

    class _getState(Interface.Method):
        name = 'getState'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.assetmgrmodel.AssetStrip.State.decode(rsp['_ret_'])
            typecheck.is_enum(_ret_, raritan.rpc.assetmgrmodel.AssetStrip.State, DecodeException)
            return _ret_

    class _getDeviceInfo(Interface.Method):
        name = 'getDeviceInfo'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.assetmgrmodel.AssetStrip.DeviceInfo.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.assetmgrmodel.AssetStrip.DeviceInfo, DecodeException)
            return _ret_

    class _getStripInfo(Interface.Method):
        name = 'getStripInfo'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.assetmgrmodel.AssetStrip.StripInfo.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.assetmgrmodel.AssetStrip.StripInfo, DecodeException)
            return _ret_

    class _getRackUnitInfo(Interface.Method):
        name = 'getRackUnitInfo'

        @staticmethod
        def encode(rackUnitNumber):
            typecheck.is_int(rackUnitNumber, AssertionError)
            args = {}
            args['rackUnitNumber'] = rackUnitNumber
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            info = raritan.rpc.assetmgrmodel.AssetStrip.RackUnitInfo.decode(rsp['info'], agent)
            typecheck.is_int(_ret_, DecodeException)
            typecheck.is_struct(info, raritan.rpc.assetmgrmodel.AssetStrip.RackUnitInfo, DecodeException)
            return (_ret_, info)

    class _getAllRackUnitInfos(Interface.Method):
        name = 'getAllRackUnitInfos'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = [raritan.rpc.assetmgrmodel.AssetStrip.RackUnitInfo.decode(x0, agent) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_struct(x0, raritan.rpc.assetmgrmodel.AssetStrip.RackUnitInfo, DecodeException)
            return _ret_

    class _getTag(Interface.Method):
        name = 'getTag'

        @staticmethod
        def encode(rackUnitNumber, slotNumber):
            typecheck.is_int(rackUnitNumber, AssertionError)
            typecheck.is_int(slotNumber, AssertionError)
            args = {}
            args['rackUnitNumber'] = rackUnitNumber
            args['slotNumber'] = slotNumber
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            tagInfo = raritan.rpc.assetmgrmodel.AssetStrip.TagInfo.decode(rsp['tagInfo'], agent)
            typecheck.is_int(_ret_, DecodeException)
            typecheck.is_struct(tagInfo, raritan.rpc.assetmgrmodel.AssetStrip.TagInfo, DecodeException)
            return (_ret_, tagInfo)

    class _getAllTags(Interface.Method):
        name = 'getAllTags'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = [raritan.rpc.assetmgrmodel.AssetStrip.TagInfo.decode(x0, agent) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_struct(x0, raritan.rpc.assetmgrmodel.AssetStrip.TagInfo, DecodeException)
            return _ret_

    class _getMainTags(Interface.Method):
        name = 'getMainTags'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = [raritan.rpc.assetmgrmodel.AssetStrip.TagInfo.decode(x0, agent) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_struct(x0, raritan.rpc.assetmgrmodel.AssetStrip.TagInfo, DecodeException)
            return _ret_

    class _getExtensionTags(Interface.Method):
        name = 'getExtensionTags'

        @staticmethod
        def encode(rackUnitNumber):
            typecheck.is_int(rackUnitNumber, AssertionError)
            args = {}
            args['rackUnitNumber'] = rackUnitNumber
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            tags = [raritan.rpc.assetmgrmodel.AssetStrip.TagInfo.decode(x0, agent) for x0 in rsp['tags']]
            typecheck.is_int(_ret_, DecodeException)
            for x0 in tags:
                typecheck.is_struct(x0, raritan.rpc.assetmgrmodel.AssetStrip.TagInfo, DecodeException)
            return (_ret_, tags)

    class _triggerPowercycle(Interface.Method):
        name = 'triggerPowercycle'

        @staticmethod
        def encode(hard):
            typecheck.is_bool(hard, AssertionError)
            args = {}
            args['hard'] = hard
            return args

        @staticmethod
        def decode(rsp, agent):
            return None

    class _programTagIDs(Interface.Method):
        name = 'programTagIDs'

        @staticmethod
        def encode(tagInfos):
            for x0 in tagInfos:
                typecheck.is_struct(x0, raritan.rpc.assetmgrmodel.AssetStrip.TagInfo, AssertionError)
            args = {}
            args['tagInfos'] = [raritan.rpc.assetmgrmodel.AssetStrip.TagInfo.encode(x0) for x0 in tagInfos]
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _getFirmwareUpdateState(Interface.Method):
        name = 'getFirmwareUpdateState'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.assetmgrmodel.AssetStrip.FirmwareUpdateState.decode(rsp['_ret_'])
            typecheck.is_enum(_ret_, raritan.rpc.assetmgrmodel.AssetStrip.FirmwareUpdateState, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(AssetStrip, self).__init__(target, agent)
        self.getState = AssetStrip._getState(self)
        self.getDeviceInfo = AssetStrip._getDeviceInfo(self)
        self.getStripInfo = AssetStrip._getStripInfo(self)
        self.getRackUnitInfo = AssetStrip._getRackUnitInfo(self)
        self.getAllRackUnitInfos = AssetStrip._getAllRackUnitInfos(self)
        self.getTag = AssetStrip._getTag(self)
        self.getAllTags = AssetStrip._getAllTags(self)
        self.getMainTags = AssetStrip._getMainTags(self)
        self.getExtensionTags = AssetStrip._getExtensionTags(self)
        self.triggerPowercycle = AssetStrip._triggerPowercycle(self)
        self.programTagIDs = AssetStrip._programTagIDs(self)
        self.getFirmwareUpdateState = AssetStrip._getFirmwareUpdateState(self)

#
# Section generated by IdlC from "AssetStripLogger.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.assetmgrmodel


# interface
class AssetStripLogger(Interface):
    idlType = "assetmgrmodel.AssetStripLogger:1.0.6"

    NO_ERROR = 0

    ERR_INVALID_PARAM = 1

    # structure
    class Info(Structure):
        idlType = "assetmgrmodel.AssetStripLogger_1_0_6.Info:1.0.0"
        elements = ["capacity", "oldestRecord", "newestRecord", "totalEventCount"]

        def __init__(self, capacity, oldestRecord, newestRecord, totalEventCount):
            typecheck.is_int(capacity, AssertionError)
            typecheck.is_int(oldestRecord, AssertionError)
            typecheck.is_int(newestRecord, AssertionError)
            typecheck.is_int(totalEventCount, AssertionError)

            self.capacity = capacity
            self.oldestRecord = oldestRecord
            self.newestRecord = newestRecord
            self.totalEventCount = totalEventCount

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                capacity = json['capacity'],
                oldestRecord = json['oldestRecord'],
                newestRecord = json['newestRecord'],
                totalEventCount = json['totalEventCount'],
            )
            return obj

        def encode(self):
            json = {}
            json['capacity'] = self.capacity
            json['oldestRecord'] = self.oldestRecord
            json['newestRecord'] = self.newestRecord
            json['totalEventCount'] = self.totalEventCount
            return json

    # enumeration
    class RecordType(Enumeration):
        idlType = "assetmgrmodel.AssetStripLogger_1_0_6.RecordType:1.0.0"
        values = ["EMPTY", "ASSET_TAG_CONNECTED", "ASSET_TAG_DISCONNECTED", "ASSET_STRIP_STATE_CHANGED"]

    RecordType.EMPTY = RecordType(0)
    RecordType.ASSET_TAG_CONNECTED = RecordType(1)
    RecordType.ASSET_TAG_DISCONNECTED = RecordType(2)
    RecordType.ASSET_STRIP_STATE_CHANGED = RecordType(3)

    # structure
    class Record(Structure):
        idlType = "assetmgrmodel.AssetStripLogger_1_0_6.Record:1.0.0"
        elements = ["timestamp", "type", "assetStripNumber", "rackUnitNumber", "rackUnitPosition", "slotNumber", "tagId", "parentBladeId", "state"]

        def __init__(self, timestamp, type, assetStripNumber, rackUnitNumber, rackUnitPosition, slotNumber, tagId, parentBladeId, state):
            typecheck.is_time(timestamp, AssertionError)
            typecheck.is_enum(type, raritan.rpc.assetmgrmodel.AssetStripLogger.RecordType, AssertionError)
            typecheck.is_int(assetStripNumber, AssertionError)
            typecheck.is_int(rackUnitNumber, AssertionError)
            typecheck.is_int(rackUnitPosition, AssertionError)
            typecheck.is_int(slotNumber, AssertionError)
            typecheck.is_string(tagId, AssertionError)
            typecheck.is_string(parentBladeId, AssertionError)
            typecheck.is_enum(state, raritan.rpc.assetmgrmodel.AssetStrip.State, AssertionError)

            self.timestamp = timestamp
            self.type = type
            self.assetStripNumber = assetStripNumber
            self.rackUnitNumber = rackUnitNumber
            self.rackUnitPosition = rackUnitPosition
            self.slotNumber = slotNumber
            self.tagId = tagId
            self.parentBladeId = parentBladeId
            self.state = state

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                timestamp = raritan.rpc.Time.decode(json['timestamp']),
                type = raritan.rpc.assetmgrmodel.AssetStripLogger.RecordType.decode(json['type']),
                assetStripNumber = json['assetStripNumber'],
                rackUnitNumber = json['rackUnitNumber'],
                rackUnitPosition = json['rackUnitPosition'],
                slotNumber = json['slotNumber'],
                tagId = json['tagId'],
                parentBladeId = json['parentBladeId'],
                state = raritan.rpc.assetmgrmodel.AssetStrip.State.decode(json['state']),
            )
            return obj

        def encode(self):
            json = {}
            json['timestamp'] = raritan.rpc.Time.encode(self.timestamp)
            json['type'] = raritan.rpc.assetmgrmodel.AssetStripLogger.RecordType.encode(self.type)
            json['assetStripNumber'] = self.assetStripNumber
            json['rackUnitNumber'] = self.rackUnitNumber
            json['rackUnitPosition'] = self.rackUnitPosition
            json['slotNumber'] = self.slotNumber
            json['tagId'] = self.tagId
            json['parentBladeId'] = self.parentBladeId
            json['state'] = raritan.rpc.assetmgrmodel.AssetStrip.State.encode(self.state)
            return json

    class _getInfo(Interface.Method):
        name = 'getInfo'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.assetmgrmodel.AssetStripLogger.Info.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.assetmgrmodel.AssetStripLogger.Info, DecodeException)
            return _ret_

    class _getRecords(Interface.Method):
        name = 'getRecords'

        @staticmethod
        def encode(id, count):
            typecheck.is_int(id, AssertionError)
            typecheck.is_int(count, AssertionError)
            args = {}
            args['id'] = id
            args['count'] = count
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            records = [raritan.rpc.assetmgrmodel.AssetStripLogger.Record.decode(x0, agent) for x0 in rsp['records']]
            typecheck.is_int(_ret_, DecodeException)
            for x0 in records:
                typecheck.is_struct(x0, raritan.rpc.assetmgrmodel.AssetStripLogger.Record, DecodeException)
            return (_ret_, records)
    def __init__(self, target, agent):
        super(AssetStripLogger, self).__init__(target, agent)
        self.getInfo = AssetStripLogger._getInfo(self)
        self.getRecords = AssetStripLogger._getRecords(self)
